from django.conf.urls import url
from django.contrib.auth.decorators import login_required

from .views import (CreateFolderView, DeleteFileView, DeleteFolderView,
                    FileManagerIndexView, GetFilesView, GetTreeView,
                    RenameFolderView, UploadFileView)

app_name = 'filemanager'

urlpatterns = [
    url(r'^$', login_required(FileManagerIndexView.as_view()), name='index'),
    url(r'^get-tree/$', login_required(GetTreeView.as_view()), name='get_tree'),
    url(r'^create-folder/$', login_required(CreateFolderView.as_view()), name='create_folder'),
    url(r'^get-files/$', login_required(GetFilesView.as_view()), name='get_files'),
    url(r'^send-file/$', login_required(UploadFileView.as_view()), name='upload_file'),
    url(r'^rename-folder/$', login_required(RenameFolderView.as_view()), name='rename_folder'),
    url(r'^delete-file/$', login_required(DeleteFileView.as_view()), name='delete_file'),
    url(r'^delete-folder/$', login_required(DeleteFolderView.as_view()), name='delete_folder'),
]



from django.contrib.auth.backends import ModelBackend

from .utils import get_user_model


class DartCMSAuthBackend(ModelBackend):
    @property
    def user_model(self):
        return get_user_model()

    def authenticate(self, username=None, password=None, **kwargs):
        try:
            user = self.user_model.objects.get(username=username, is_active=True, is_staff=True)
            if user.check_password(password):
                return user
        except self.user_model.DoesNotExist:
            return None

    def get_user(self, user_id):
        try:
            return self.user_model.objects.get(pk=user_id)
        except self.user_model.DoesNotExist:
            return None



from datetime import datetime

from dartcms.apps.ads.models import AdPlace
from dartcms.utils.loading import get_model
from django import template

register = template.Library()


@register.inclusion_tag('dartcms/apps/ads/place.html', name='ad_place', takes_context=True)
def ad_place(context, place, limit=1):
    return get_ads(context, place, limit)


def get_ads(context, place, limit):
    if not hasattr(context['request'], 'page') or context['request'].page is None:
        return {}

    section = context['request'].page.ad_section
    place = AdPlace.objects.get(slug=place)

    ad_model = get_model('ads', 'Ad')

    ads = ad_model.objects.filter(
        section=section,
        place=place,
        date_from__lt=datetime.now(),
        date_to__gt=datetime.now()
    ).order_by('?')[0:limit]

    return {'object_list': ads}



from dartcms.utils.loading import get_model
from dartcms.views import GridView, InsertObjectView, JSONView
from dartcms.views.mixins import JSONResponseMixin, ModulePermissionsMixin
from django.http import Http404
from django.utils.translation import ugettext as __


class GetTreeView(GridView, JSONResponseMixin):
    def post(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def render_to_response(self, context, **response_kwargs):
        return self.render_to_json_response(context, safe=False, **response_kwargs)

    def get_data(self, context):
        sections = self.object_list.filter(parent=None)
        tree = [dict(pk='0', parent_id=None, name=__('Root section'), is_visible='', children=[])]
        for section in sections:
            tree[0]['children'].append(section.serializable_object())
        return tree


class InsertSectionView(InsertObjectView):
    def get_initial(self):
        return {'parent': self.model.objects.filter(pk=self.request.GET.get('parent', 0)).first()}


class TreeActionView(ModulePermissionsMixin, JSONView):
    def get_sections(self):
        target_id = self.request.GET.get('target')
        source_id = self.request.GET.get('source')

        if target_id and source_id:
            ProductSection = get_model('shop', 'ProductSection')

            target = ProductSection.objects.get(pk=target_id) if target_id != '0' else None
            source = ProductSection.objects.get(pk=source_id)

            return target, source

        raise Http404


class AppendSectionView(TreeActionView):
    def get_data(self, context):
        target, source = self.get_sections()
        source.move_to(target, position='last-child')
        return {'result': True}


class MoveSectionView(TreeActionView):
    def get_data(self, context):
        position = self.request.GET.get('position')
        target, source = self.get_sections()

        if target:
            source.move_to(target, position=position)
            return {'result': True}
        return {'result': False}



# Generated by Django 1.9.7 on 2016-07-13 03:49
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('feedback', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='questionmessage',
            name='answer',
            field=models.TextField(blank=True, null=True, verbose_name='Answer'),
        ),
        migrations.AddField(
            model_name='questionmessage',
            name='is_visible',
            field=models.BooleanField(default=False, verbose_name='Is Visible'),
        ),
    ]



# Generated by Django 1.9.7 on 2016-07-12 15:16
from __future__ import unicode_literals

from django.db import migrations
from django.utils import translation
from django.utils.translation import gettext_lazy as _

sitesettings = [
    {
        'slug': 'site_name',
        'description': _('Site name'),
        'type': 'text'
    },
    {
        'slug': 'email_for_feedback',
        'description': _('Email for feedback'),
        'type': 'text'
    },
]


def insert_settings(apps, schema):
    from django.conf import settings
    translation.activate(settings.LANGUAGE_CODE)

    SiteSettings = apps.get_model('sitesettings', 'SiteSettings')
    for sitesetting in sitesettings:
        SiteSettings.objects.create(**sitesetting)

    translation.deactivate()


def delete_settings(apps, schema):
    SiteSettings = apps.get_model('sitesettings', 'SiteSettings')
    for sitesetting in sitesettings:
        SiteSettings.objects.get(slug=sitesetting['slug']).delete()


class Migration(migrations.Migration):
    dependencies = [
        ('sitesettings', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(insert_settings, delete_settings)
    ]



# Generated by Django 1.9.6 on 2016-07-06 08:24
from __future__ import unicode_literals

import datetime

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Feed',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('slug', models.SlugField(verbose_name='URL')),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='FeedItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=1024, verbose_name='Title')),
                ('short_text', models.TextField(verbose_name='Short Text')),
                ('full_text', models.TextField(verbose_name='Full Text')),
                ('picture', models.ImageField(upload_to=b'feeds/%Y/%m/%d', verbose_name='Picture')),
                ('seo_keywords', models.TextField(blank=True, help_text='Do not use more than 255 symbols', verbose_name='Keywords')),
                ('seo_description', models.TextField(blank=True, help_text='Do not use more than 1024 symbols', verbose_name='Description')),
                ('is_visible', models.BooleanField(default=True, verbose_name='Show on Site')),
                ('date_published', models.DateTimeField(default=datetime.datetime.now, verbose_name='Date of Publication')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date of Creation')),
                ('feed', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='feeds.Feed', verbose_name='Feed')),
            ],
            options={
                'ordering': ['-date_published'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='FeedType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('slug', models.SlugField(verbose_name='Slug')),
            ],
        ),
        migrations.AddField(
            model_name='feed',
            name='type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='feeds', to='feeds.FeedType', verbose_name='Type'),
        ),
        migrations.AlterUniqueTogether(
            name='feed',
            unique_together=set([('type', 'slug')]),
        ),
    ]



from dartcms.utils.loading import is_model_registered

from .abstract_models import *

__all__ = [
    'FormType',
]

if not is_model_registered('feedback', 'QuestionMessage'):
    class QuestionMessage(AbstractQuestionMessage):
        pass

    __all__.append('QuestionMessage')

if not is_model_registered('feedback', 'ContactMessage'):
    class ContactMessage(AbstractContactMessage):
        pass

    __all__.append('ContactMessage')



from dartcms import get_model
from django.http.response import Http404
from django.shortcuts import redirect


class PageMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        return response

    def process_request(self, request):
        PageModule = get_model('pages', 'PageModule')
        Page = get_model('pages', 'Page')

        request_path = request.path.strip('/')
        path_parts = request_path.split('/')

        path = '/' if request_path == '' else '/%s/' % request.path.strip('/')
        module_slug = 'homepage' if path == '/' else path_parts[0]

        try:
            request.page_module = PageModule.objects.get(slug=module_slug)
        except PageModule.DoesNotExist:
            return

        page = None

        try:
            page = Page.objects.get(url=path)
        except Page.DoesNotExist:
            while path_parts:
                path_parts.pop()
                if path_parts:
                    path = '/%s/' % '/'.join(path_parts)
                    try:
                        page = Page.objects.get(url=path)
                        break
                    except Page.DoesNotExist:
                        continue

        if page:
            if page.redirect_url:
                return redirect(page.redirect_url)
            request.page = page
            request.page_children = page.children.filter(is_enabled=True, is_in_menu=True)

    def process_view(self, request, view_func, view_args, view_kwargs):
        namespaces = request.resolver_match.namespaces
        if 'dartcms' in namespaces or 'admin' in namespaces:
            return

        if hasattr(request, 'page_module') and not hasattr(request, 'page'):
            raise Http404('Attribute "page" is not in request')

        return self.process_request(request)



from dartcms.utils.loading import get_model
from django.forms import HiddenInput
from django.utils.translation import ugettext_lazy as _
from django.forms import ModelForm


class ProductSectionForm(ModelForm):
    class Meta:
        model = get_model('shop', 'ProductSection')
        exclude = ['catalog', 'sort']

        widgets = {'parent': HiddenInput()}

        fieldsets = (
            ('main', {'fields': (
                'parent',
                'name',
                'description',
            ), 'legend': _('Main')}),
            ('seo', {'fields': (
                'seo_keywords',
                'seo_description',
            ), 'legend': _('SEO')}),
            ('advanced', {'fields': (
                'image',
                'is_visible',
            ), 'legend': _('Advanced')}))



import json
from datetime import datetime

from django.db.models import ProtectedError
from django.views.generic import ListView, CreateView, UpdateView, DeleteView, TemplateView
from django.utils.translation import ugettext as _

from extra_views import CreateWithInlinesView, UpdateWithInlinesView

from dartcms.utils.db import get_model_field_type, get_model_field_label, get_model_field
from dartcms.utils.forms import DynamicFieldsForm
from dartcms.utils.serialization import DartCMSSerializer
from dartcms.utils.translation import get_date_format
from .mixins import AdminMixin, JSONResponseMixin


class JSONView(JSONResponseMixin, TemplateView):
    def post(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def render_to_response(self, context, **response_kwargs):
        return self.render_to_json_response(context, **response_kwargs)


class GridView(AdminMixin, JSONResponseMixin, ListView):
    template_name = 'dartcms/views/grid.html'
    paginate_by = None
    allow_empty = True
    grid_columns = [
        {'field': 'name', 'width': '70%', 'label': _('Name')},
        {'field': 'date_created', 'width': '30%', 'label': _('Date created'), 'type': 'DATETIME'},
    ]
    search = None
    search_form = None
    base_grid_actions = ['insert', 'update', 'delete']
    additional_grid_actions = []
    model_properties = []
    parent_str = ''
    single_select = True

    def get_search_form_kwargs(self):
        kwargs = {'auto_id': 'id_for_search_%s'}
        if 'search' in self.request.GET:
            kwargs['initial'] = self.request.GET
        return kwargs

    def get_search_form(self):
        if self.search:
            form_kwargs = self.get_search_form_kwargs()

            class SearchForm(DynamicFieldsForm):
                pass

            fields = []
            for item in self.search:
                label = None

                if isinstance(item, dict):
                    label = item.get('label')
                    if 'field' in item:
                        field = item['field']
                    else:
                        raise KeyError('Search dict item does not have "field" key.')
                else:
                    field = item

                field_type = get_model_field_type(self.model, field)
                kwargs = {
                    'label': label if label else get_model_field_label(self.model, field),
                    'required': False
                }

                if field_type == 'FOREIGN_KEY':
                    kwargs['queryset'] = get_model_field(self.model, field).related_model.objects.all()

                fields.append({'field': field, 'type': field_type, 'kwargs': kwargs})

            form_kwargs['extra'] = fields
            return SearchForm(**form_kwargs)

    def filter_queryset(self, queryset):
        for field in self.search:

            if isinstance(field, dict):
                if 'field' in field:
                    field = field['field']
                else:
                    raise KeyError('Search dict item does not have "field" key.')

            field_type = get_model_field_type(self.model, field)
            if field_type in ('DATETIME', 'DATE', 'TIME'):
                date_from = self.request.GET.get('%s_from' % field, '')
                date_to = self.request.GET.get('%s_to' % field, '')
                if date_from and date_to:
                    queryset = queryset.filter(**{
                        '%s__gte' % field: datetime.strptime(date_from, '%s %%H:%%M:%%S' % get_date_format()),
                        '%s__lte' % field: datetime.strptime(date_to, '%s %%H:%%M:%%S' % get_date_format()),
                    })
            elif field_type == 'BOOLEAN':
                term = self.request.GET.get(field, False)
                if term:
                    queryset = queryset.filter(**{
                        field: True if term == 'ON' else False
                    })
            elif field_type == 'FOREIGN_KEY':
                term = self.request.GET.get(field, '')
                if term:
                    queryset = queryset.filter(**{
                        '%s_id' % field: term
                    })
            else:
                term = self.request.GET.get(field, '')
                if term:
                    queryset = queryset.filter(**{
                        '%s__icontains' % field: term
                    })

        return queryset

    def get_paginate_by(self, queryset):
        return self.request.GET.get('rows', self.paginate_by)

    def get_queryset(self):
        if self.parent_kwarg_name:
            kwarg = self.kwargs[self.parent_kwarg_name]

            if self.parent_model_fk is not None:
                fk = self.parent_model_fk
            else:
                fk = '%s_id' % self.parent_model.__name__.lower()

            self.parent_str = self.model._meta.get_field(fk).related_model.objects.get(pk=kwarg)

            queryset = self.model.objects.filter(**{
                '%s__exact' % fk: kwarg
            })
        else:
            queryset = super(GridView, self).get_queryset()

        if self.search and 'search' in self.request.GET:
            queryset = self.filter_queryset(queryset)

        sort = self.request.GET.get('sort')
        order = self.request.GET.get('order')
        if sort and order:
            sort = sort if order == 'asc' else '-%s' % sort
            queryset = queryset.order_by(sort)

        return queryset

    def get_total_rows_count(self):
        queryset = self.get_queryset()
        return queryset.count()

    def get_grid_actions(self):
        perms = self.user_module_permissions
        grid_actions = []

        for action in ['insert', 'update', 'delete']:
            if getattr(perms, 'can_%s' % action, False) and action in self.base_grid_actions:
                grid_actions.append(action)

        return grid_actions

    def get_additional_grid_actions(self):
        perms = self.user_module_permissions
        if not perms:
            return []

        additional_actions = []

        for additional_action in self.additional_grid_actions:
            required_perms = additional_action.get('required_permissions')
            if required_perms:
                if required_perms == '__all__':
                    if all([perms.can_insert, perms.can_update, perms.can_delete]):
                        additional_actions.append(additional_action)
                else:
                    for perm in required_perms:
                        has_perm = getattr(perms, 'can_%s' % perm, False)
                        if has_perm:
                            additional_actions.append(additional_action)
            else:
                additional_actions.append(additional_action)

        return additional_actions

    def get_context_data(self, **kwargs):
        context = super(GridView, self).get_context_data(**kwargs)
        context.update({
            'grid_columns': self.grid_columns,
            'grid_actions': self.get_grid_actions(),
            'additional_grid_actions': self.get_additional_grid_actions(),
            'search_form': self.get_search_form(),
            'date_format': get_date_format(),
            'urls_kwargs': self.kwargs,
            'parent_str': self.parent_str,
            'single_select': self.single_select
        })
        return context

    def render_to_response(self, context, **response_kwargs):
        if self.request.is_ajax():
            return self.render_to_json_response(context, **response_kwargs)
        return super(GridView, self).render_to_response(context, **response_kwargs)

    def get_data(self, context):
        queryset = self.get_queryset()
        page_size = self.get_paginate_by(queryset)
        if page_size:
            paginator, page, queryset, is_paginated = self.paginate_queryset(queryset, page_size)
        return {
            'total': self.get_total_rows_count(),
            'rows': json.loads(DartCMSSerializer().serialize(
                queryset=queryset,
                props=self.model_properties
            ))  # TODO: refactor.
        }


class InlineErrorsMixin(object):
    @staticmethod
    def get_inline_errors(inlines):
        errors = {}

        for inline in inlines:
            for form in inline:
                for field_name, field_errors in form.errors.items():
                    field = form[field_name]
                    errors[field.html_name] = field_errors

        return errors


class AjaxInsertObjectMixin(AdminMixin, JSONResponseMixin):
    template_name = 'dartcms/views/insert.html'

    def save_object(self, form, **kwargs):
        self.object = form.save(commit=False)
        if self.parent_kwarg_name:
            setattr(self.object, self.get_foreign_key_name(), self.kwargs[self.parent_kwarg_name])
        self.object.save()
        form.save_m2m()

        inlines = kwargs.pop('inlines', [])
        for formset in inlines:
            formset.save()

        return self.object

    def get_initial(self):
        initial = super(AjaxInsertObjectMixin, self).get_initial()
        if self.parent_model_fk and self.parent_kwarg_name in self.kwargs:
            initial.update({
                self.parent_model_fk: self.kwargs[self.parent_kwarg_name]
            })
        return initial


class InsertObjectView(AjaxInsertObjectMixin, CreateView):
    def form_valid(self, form):
        self.save_object(form)
        return self.render_to_json_response({'result': True, 'action': 'INSERT'})

    def form_invalid(self, form):
        return self.render_to_json_response({'result': False, 'errors': form.errors})


class InsertObjectWithInlinesView(InlineErrorsMixin,
                                  AjaxInsertObjectMixin,
                                  CreateWithInlinesView):

    def forms_valid(self, form, inlines):
        self.save_object(form, inlines=inlines)
        return self.render_to_json_response({'result': True, 'action': 'INSERT'})

    def forms_invalid(self, form, inlines):
        errors = form.errors.copy()
        errors.update(self.get_inline_errors(inlines))
        return self.render_to_json_response({'result': False, 'errors': errors})


class AjaxUpdateObjectMixin(AdminMixin, JSONResponseMixin):
    template_name = 'dartcms/views/update.html'


class UpdateObjectView(AjaxUpdateObjectMixin, UpdateView):
    def get_context_data(self, *args, **kwargs):
        context = super(UpdateObjectView, self).get_context_data(*args, **kwargs)
        if not self.user_module_permissions.can_update:
            context['read_only'] = True
        return context

    def form_valid(self, form):
        form.save()
        return self.render_to_json_response({'result': True, 'action': 'UPDATE'})

    def form_invalid(self, form):
        return self.render_to_json_response({'result': False, 'errors': form.errors})


class UpdateObjectWithInlinesView(InlineErrorsMixin,
                                  AjaxUpdateObjectMixin,
                                  UpdateWithInlinesView):

    def forms_valid(self, form, inlines):
        form.save()
        for formset in inlines:
            formset.save()
        return self.render_to_json_response({'result': True, 'action': 'UPDATE'})

    def forms_invalid(self, form, inlines):
        errors = form.errors.copy()
        errors.update(self.get_inline_errors(inlines))
        return self.render_to_json_response({'result': False, 'errors': errors})


class DeleteObjectView(AdminMixin, JSONResponseMixin, DeleteView):
    template_name = 'dartcms/views/delete.html'

    def delete(self, request, *args, **kwargs):
        self.object = self.get_object()
        try:
            self.object.delete()
        except ProtectedError:
            return self.render_to_json_response({'result': False, 'error': 'PROTECTED'})
        return self.render_to_json_response({'result': True, 'action': 'DELETE'})


class DeleteMultipleObjectView(AdminMixin, JSONResponseMixin, TemplateView):
    template_name = 'dartcms/views/delete.html'
    kwarg_name = 'pks'

    def get(self, request, *args, **kwargs):
        pks = kwargs.get(self.kwarg_name).split(',')
        context = self.get_context_data(count_records=len(pks))
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        return self.delete(request, *args, **kwargs)

    def delete(self, request, *args, **kwargs):
        pks = kwargs.get(self.kwarg_name).split(',')
        try:
            self.model.objects.filter(pk__in=pks).delete()
        except ProtectedError:
            return self.render_to_json_response({'result': False, 'error': 'PROTECTED'})
        return self.render_to_json_response({'result': True, 'action': 'DELETE'})



# Generated by Django 1.9.6 on 2016-05-20 02:26
from __future__ import unicode_literals

from django.db import migrations
from django.utils import translation
from django.utils.translation import gettext_lazy as _

MODULE_GROUPS = [
    {
        'sort': 4,
        'description': '',
        'fa': 'fa-rocket',
        'slug': 'ads',
        'name': _('Advertising'),
        'modules': [
            {
                'sort': 1,
                'is_enabled': True,
                'name': _('Ad banners'),
                'slug': 'ad',
                'description': ''
            },
            {
                'sort': 2,
                'is_enabled': True,
                'name': _('Ad places'),
                'slug': 'adplace',
                'description': ''
            },
            {
                'sort': 3,
                'is_enabled': True,
                'name': _('Ad sections'),
                'slug': 'adsection',
                'description': ''
            }
        ]
    }
]


def insert_modules(apps, schema):
    from django.conf import settings
    translation.activate(settings.LANGUAGE_CODE)

    ModuleGroup = apps.get_model('modules', 'ModuleGroup')
    Module = apps.get_model('modules', 'Module')

    for group in MODULE_GROUPS:
        group_modules = group.pop('modules', [])
        group = ModuleGroup.objects.create(**group)
        for module in group_modules:
            module['group'] = group
            Module.objects.create(**module)

    translation.deactivate()


def delete_modules(apps, schema):
    ModuleGroup = apps.get_model('modules', 'ModuleGroup')

    for group in MODULE_GROUPS:
        ModuleGroup.objects.get(slug=group['slug']).delete()


class Migration(migrations.Migration):
    dependencies = [
        ('modules', '0004_insert_feedback_module'),
    ]

    operations = [
        migrations.RunPython(insert_modules, delete_modules)
    ]



from dartcms import get_model
from dartcms.utils.config import DartCMSConfig
from django.utils.translation import ugettext_lazy as _

from .forms import OrderForm

app_name = 'orders'

Order = get_model('shop', 'Order')

config = DartCMSConfig({
    'model': Order,
    'grid': {
        'grid_columns': [
            {'field': 'order_number', 'width': '10%', 'label': _('#')},
            {'field': 'date_created', 'width': '20%'},
            {'field': 'fullname', 'width': '30%'},
            {'field': 'status', 'width': '10%'},
            {'field': 'shipping_type', 'width': '10%'},
            {'field': 'payment_type', 'width': '10%'},
            {'field': 'total', 'width': '10%', 'label': _('Total')}
        ],
        'search': [
            'fullname', 'date_created'
        ],
        'base_grid_actions': ['update'],
        'additional_grid_actions': [
            {'url': 'details', 'label': _('Details'),
             'kwarg_name': 'order', 'include_urls': 'dartcms.apps.shop.order_detail.urls'}
        ],
        'model_properties': ['order_number', 'total']
    },
    'form': {
        'form_class': OrderForm
    }
})

urlpatterns = config.get_urls(exclude=['insert', 'delete'])



from dartcms.apps.auth.utils import get_user_model
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from .managers import CMSUserManager

User = get_user_model()


class UserGroup(models.Model):
    class Meta:
        verbose_name = _('User group')
        verbose_name_plural = _('User groups')
        ordering = ['name']

    name = models.CharField(max_length=64, verbose_name=_('Group name'))
    users = models.ManyToManyField(settings.AUTH_USER_MODEL, related_name='user_groups')

    def __str__(self):
        return self.name


class CMSUser(User):
    class Meta:
        verbose_name = _('CMS User')
        verbose_name_plural = _('CMS Users')
        proxy = True

    def save(self, *args, **kwargs):
        self.is_staff = True
        super(CMSUser, self).save(*args, **kwargs)

    objects = CMSUserManager()



# Generated by Django 1.10.4 on 2016-12-14 05:12
from __future__ import unicode_literals

from dartcms.apps.users.models import UserGroup
from django.db import migrations


def add_user_group(apps, schema):
    UserGroup.objects.create(name='Users')


class Migration(migrations.Migration):
    dependencies = [
        ('users', '0002_auto_20161214_0258'),
    ]

    operations = [
        migrations.RunPython(add_user_group)
    ]



from django import forms
from django.utils.translation import ugettext_lazy as _

BOOLEAN_CHOICES = (
    ('', ''),
    ('ON', _('Yes')),
    ('OFF', _('No'))
)

DYNAMIC_FIELDS_MAPPING = {
    'STRING': {'class': forms.CharField, 'defaults': {'max_length': 255}},
    'DATE': {'class': forms.DateField, 'defaults': {'widget': forms.TextInput(attrs={'class': 'date'})}},
    'TIME': {'class': forms.TimeField, 'defaults': {'widget': forms.TextInput(attrs={'class': 'time'})}},
    'DATETIME': {'class': forms.DateTimeField, 'defaults': {'widget': forms.TextInput(attrs={'class': 'datetime'})}},
    'FOREIGN_KEY': {'class': forms.ModelChoiceField},
    'BOOLEAN': {'class': forms.CharField,
                'defaults': {'widget': forms.Select(choices=BOOLEAN_CHOICES)}}
}


class DynamicFieldsForm(forms.Form):
    def __init__(self, *args, **kwargs):
        extra = kwargs.pop('extra', [])

        super(DynamicFieldsForm, self).__init__(*args, **kwargs)

        for item in extra:
            field = DYNAMIC_FIELDS_MAPPING.get(item['type'], DYNAMIC_FIELDS_MAPPING['STRING'])
            if 'defaults' in field:
                defaults = field['defaults'].copy()
                defaults.update(item['kwargs'])
            else:
                defaults = item['kwargs']

            if item['type'] in ('DATE', 'TIME', 'DATETIME'):
                label = defaults['label']
                for suffix in ('from', 'to'):
                    defaults['label'] = '%s (%s)' % (label, _('from') if suffix == 'from' else _('to'))
                    self.fields['%s_%s' % (item['field'], suffix)] = field['class'](**defaults)
            else:
                self.fields[item['field']] = field['class'](**defaults)



# Generated by Django 1.9.9 on 2016-10-26 07:11
from __future__ import unicode_literals

import dartcms.utils.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0007_feedback_pagemodule'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='page',
            managers=[
            ],
        ),
        migrations.AlterField(
            model_name='page',
            name='after_content',
            field=dartcms.utils.fields.RteField(blank=True, default=b'', verbose_name='After Content'),
        ),
        migrations.AlterField(
            model_name='page',
            name='before_content',
            field=dartcms.utils.fields.RteField(blank=True, default=b'', verbose_name='Before Content'),
        ),
        migrations.AlterField(
            model_name='pagemodule',
            name='related_model',
            field=models.CharField(blank=True, default=None, max_length=32, null=True, verbose_name='Related Model'),
        ),
    ]



import os

from dartcms.views import JSONView
from django.conf import settings
from django.http import Http404
from django.utils import formats
from django.views.generic import TemplateView

from .forms import FolderForm
from .models import File, Folder


class FileManagerIndexView(TemplateView):
    template_name = 'dartcms/apps/filemanager/index.html'


class GetTreeView(JSONView):
    def get_data(self, context):
        response = []

        for folder in Folder.objects.all().order_by('name'):
            response.append({
                'id': folder.id,
                'text': folder.name,
                'iconCls': 'icon-folder'
            })
        return response


class CreateFolderView(JSONView):
    def get_data(self, context):
        folder_form = FolderForm(self.request.POST)
        if folder_form.is_valid():
            folder_form.save()
            response = {'result': True}

        else:
            response = {
                'result': False,
                'errors': folder_form.errors
            }
        return response


class GetFilesView(JSONView):
    def get_data(self, context):
        try:
            folder = Folder.objects.get(pk=self.request.GET.get('folder_id'))
            response = []
            for f in folder.files.all().order_by('-date_created'):
                response.append({
                    'id': f.id,
                    'name': os.path.basename(f.path.name),
                    'path': settings.MEDIA_URL + f.path.name,
                    'date_created': formats.date_format(f.date_created, "SHORT_DATETIME_FORMAT")
                })
            return response
        except Folder.DoesNotExist:
            raise Http404


class UploadFileView(JSONView):
    def get_data(self, context):
        try:
            folder_id = int(self.request.POST.get('folder_id'))
            folder = Folder.objects.get(id=folder_id)
            for f in self.request.FILES.getlist('files'):
                File(folder=folder, path=f).save()

            return {'result': True}
        except Folder.DoesNotExist:
            raise Http404


class RenameFolderView(JSONView):
    def get_data(self, context):
        try:
            folder = Folder.objects.get(pk=self.request.GET.get('folder_id'))
            folder_form = FolderForm(self.request.POST, instance=folder)
            if folder_form.is_valid():
                folder_form.save()
                response = {'result': 'success'}
            else:
                response = {
                    'result': 'error',
                    'errors': folder_form.errors
                }
            return response
        except Folder.DoesNotExist:
            raise Http404


class DeleteFolderView(JSONView):
    def get_data(self, context):
        try:
            Folder.objects.get(pk=self.request.GET.get('folder_id')).delete()
            return {'result': 'success'}
        except Folder.DoesNotExist:
            raise Http404


class DeleteFileView(JSONView):
    def get_data(self, context):
        try:
            File.objects.get(pk=self.request.GET.get('file_id'), folder_id=self.request.GET.get('folder_id')).delete()
            return {'result': 'success'}
        except Folder.DoesNotExist:
            raise Http404



# Generated by Django 1.10.4 on 2017-08-29 05:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('feedback', '0003_auto_20160801_1607'),
    ]

    operations = [
        migrations.AlterField(
            model_name='formtype',
            name='slug',
            field=models.SlugField(choices=[('contact', 'Contact form'), ('question', 'Questions form')], verbose_name='Type ID'),
        ),
    ]



from dartcms.apps.users.views import \
    ChangePasswordView as CMSChangePasswordView
from django.urls import reverse_lazy


class ChangePasswordView(CMSChangePasswordView):
    success_url = reverse_lazy('dartcms:siteusers:index')



from django.contrib.auth.mixins import UserPassesTestMixin


class StaffRequiredMixin(UserPassesTestMixin):
    login_url = 'dartcms:auth:login'

    def test_func(self):
        return self.request.user.is_staff



from django.urls import NoReverseMatch, reverse

from .models import Module, ModulePermission


def get_current_module(path):
    try:
        dartcms_path = reverse('dartcms:dashboard:index')
    except NoReverseMatch:
        return {}

    module_slug = path.replace(dartcms_path, '', 1).split('/')[0]
    if module_slug:
        try:
            return Module.objects.get(slug=module_slug)
        except Module.DoesNotExist:
            pass


def get_current_module_perms(request):
    cms_module = get_current_module(request.path)
    if cms_module:
        return ModulePermission.objects.filter(user=request.user, module=cms_module).first()



# Generated by Django 1.10.4 on 2017-08-29 05:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filemanager', '0002_auto_20160711_1627'),
    ]

    operations = [
        migrations.AlterField(
            model_name='file',
            name='path',
            field=models.FileField(max_length=300, upload_to='uploads/%Y/%m/%d'),
        ),
        migrations.AlterField(
            model_name='folder',
            name='name',
            field=models.CharField(error_messages={'unique': 'Folder with this name is already exists'}, max_length=255, unique=True),
        ),
    ]



import json
import os
import types
import copy
from collections import OrderedDict

from django import template
from django.forms import (CheckboxInput, CheckboxSelectMultiple,
                          ClearableFileInput, DateInput, DateTimeInput,
                          HiddenInput, RadioSelect, SelectMultiple)
from widget_tweaks.templatetags.widget_tweaks import (append_attr,
                                                      silence_without_field)

from .ifstartswith import do_startswith

try:
    import Image, ImageOps
except ImportError:
    from PIL import Image, ImageOps

register = template.Library()


#######################################################################################
# Startswith template tags.                                                           #
# Usage:                                                                              #
# {% ifstartswith foo 'bar' %}                                                        #
# ...                                                                                 #
# {% endifstartswith %}                                                               #
#######################################################################################


@register.tag
def ifstartswith(parser, token):
    return do_startswith(parser, token, False)


@register.tag
def ifnotstartswith(parser, token):
    return do_startswith(parser, token, True)


#######################################################################################
# Thumbnail filters.                                                                  #
#######################################################################################


@register.filter
def thumbnail(file, size='220x220'):
    x, y = [int(x) for x in size.split('x')]
    if isinstance(file, str):
        file = open(file, 'rw')

    filehead, filetail = os.path.split(file.path)
    basename, format = os.path.splitext(filetail)
    miniature = basename + '_' + size + format
    filename = file.path
    miniature_filename = os.path.join(filehead, miniature)
    filehead, filetail = os.path.split(file.url)
    miniature_url = filehead + '/' + miniature
    if os.path.exists(miniature_filename) and os.path.getmtime(filename) > os.path.getmtime(miniature_filename):
        os.unlink(miniature_filename)
        # if the image wasn't already resized, resize it
    if not os.path.exists(miniature_filename) and os.path.exists(filename):
        image = Image.open(filename)

        if (image.size[0] < x):
            x = image.size[0]
        if (image.size[1] < y):
            y = image.size[1]

        img_ratio = float(image.size[0]) / image.size[1]
        # resize but constrain proportions?
        if x == 0.0:
            x = y * img_ratio
        elif y == 0.0:
            y = x / img_ratio

        thumb_ratio = float(x) / y
        x = int(x);
        y = int(y)

        if (img_ratio > thumb_ratio):
            c_width = x * image.size[1] / y
            c_height = image.size[1]
            originX = image.size[0] / 2 - c_width / 2
            originY = 0
        else:
            c_width = image.size[0]
            c_height = y * image.size[0] / x
            originX = 0
            originY = image.size[1] / 2 - c_height / 2

        cropBox = (originX, originY, originX + c_width, originY + c_height)
        image = image.crop(cropBox)
        image.thumbnail([x, y], Image.ANTIALIAS)
        try:
            image.save(miniature_filename, image.format, quality=90, optimize=1)
        except:
            image.save(miniature_filename, image.format, quality=90)

    return miniature_url


@register.filter
def thumbnail_with_max_side(file, size='220'):
    filehead, filetail = os.path.split(file.path)
    basename, format = os.path.splitext(filetail)
    miniature = basename + '_' + size + format
    filename = file.path
    miniature_filename = os.path.join(filehead, miniature)
    filehead, filetail = os.path.split(file.url)
    miniature_url = filehead + '/' + miniature
    if os.path.exists(miniature_filename) and os.path.getmtime(filename) > os.path.getmtime(miniature_filename):
        os.unlink(miniature_filename)
    if not os.path.exists(miniature_filename) and os.path.exists(filename):
        image = Image.open(filename)
        x, y = 0, 0

        if (image.size[0] < x) or (image.size[1] < y):
            return filehead + '/' + basename + format

        img_ratio = float(image.size[0]) / image.size[1]
        if img_ratio > 1:
            x = int(size)
        else:
            y = int(size)
        if x == 0.0:
            x = y * img_ratio
        elif y == 0.0:
            y = x / img_ratio

        thumb_ratio = float(x) / y

        x = int(x)
        y = int(y)

        if (img_ratio > thumb_ratio):
            c_width = x * image.size[1] / y
            c_height = image.size[1]
            originX = image.size[0] / 2 - c_width / 2
            originY = 0
            cropBox = (originX, originY, originX + c_width, originY + c_height)
            image = image.crop(cropBox)
        else:
            c_width = image.size[0]
            c_height = y * image.size[0] / x
            originX = 0
            originY = image.size[1] / 2 - c_height / 2
            cropBox = (originX, originY, originX + c_width, originY + c_height)
            image = image.crop(cropBox)

        image.thumbnail([x, y], Image.ANTIALIAS)
        try:
            image.save(miniature_filename, image.format, quality=90, optimize=1)
        except:
            image.save(miniature_filename, image.format, quality=90)

    return miniature_url


#######################################################################################
# File-related filters.                                                               #
#######################################################################################


@register.filter
def filename(value):
    return os.path.basename(str(value))


@register.filter
def ext(value):
    name, extension = os.path.splitext(value.name.lower())
    return extension.replace('.', '')


#######################################################################################
# Attribute lookup filters                                                            #
#######################################################################################


@register.filter
def lookup(d, key):
    return d[key]


@register.filter
def attribute(obj, attribute_name):
    return getattr(obj, attribute_name, None)


#######################################################################################
# Form helpers.                                                                       #
#######################################################################################


@register.filter
def is_checkbox(field):
    return field.field.widget.__class__.__name__ == CheckboxInput().__class__.__name__


@register.filter
def is_hidden(field):
    return field.field.widget.__class__.__name__ == HiddenInput().__class__.__name__


@register.filter
def is_file(field):
    return field.field.widget.__class__.__name__ == ClearableFileInput().__class__.__name__


@register.filter
def is_radio(field):
    return field.field.widget.__class__.__name__ == RadioSelect().__class__.__name__


@register.filter
def is_date(field):
    return field.field.widget.__class__.__name__ == DateInput().__class__.__name__


@register.filter
def is_datetime(field):
    return field.field.widget.__class__.__name__ == DateTimeInput().__class__.__name__


@register.filter
def is_multiple_select(field):
    return field.field.widget.__class__.__name__ == SelectMultiple().__class__.__name__


@register.filter
def is_checkbox_multiple_select(field):
    return field.field.widget.__class__.__name__ == CheckboxSelectMultiple().__class__.__name__


@register.filter
def test_widget_class(field):
    return field.field.widget.__class__.__name__


#######################################################################################
# Widget tweaks backport.                                                             #
#######################################################################################


def _process_field_attributes(field, attr, process):
    params = attr.split(':', 1)
    attribute = params[0]
    value = params[1] if len(params) == 2 else ''

    old_as_widget = field.as_widget

    def as_widget(self, widget=None, attrs=None, only_initial=False):
        attrs = attrs or {}
        process(widget or self.field.widget, attrs, attribute, value)
        html = old_as_widget(widget, attrs, only_initial)
        self.as_widget = old_as_widget
        return html

    field.as_widget = types.MethodType(as_widget, field)
    return field


@register.filter('attr')
@silence_without_field
def set_attr(field, attr):
    def process(widget, attrs, attribute, value):
        attrs[attribute] = value

    return _process_field_attributes(field, attr, process)


@register.filter('add_error_attr')
@silence_without_field
def add_error_attr(field, attr):
    if hasattr(field, 'errors') and field.errors:
        return set_attr(field, attr)
    return field


@register.filter('add_class')
@silence_without_field
def add_class(field, css_class):
    return append_attr(field, 'class:' + css_class)


#######################################################################################
# Tinymce settings                                                                    #
#######################################################################################
@register.simple_tag
def get_tinymce_settings():
    from django.conf import settings
    return getattr(settings, 'DARTCMS_TINYMCE_SETTINGS', {})


#######################################################################################
# Useful utils                                                                        #
#######################################################################################
@register.filter('get_type')
def get_type(value):
    mapping = {
        str: 'str',
        dict: 'dict',
        list: 'list',
        tuple: 'tuple',
        bool: 'bool',
        int: 'int'
    }
    return mapping.get(type(value))


@register.filter('json_loads')
def json_loads(value):
    return json.loads(value)


@register.filter('json_dumps')
def json_dumps(value):
    return json.dumps(value)


@register.simple_tag(name='fieldset_form')
def fieldset_form(form):

    if hasattr(form.Meta, 'fieldsets'):
        res = {'fieldsets': []}
        for fieldset in form.Meta.fieldsets:

            new_form = copy.copy(form)
            new_form.fields = OrderedDict(
                [(key, value) for key, value in form.fields.items()
                 if key in fieldset[1]['fields']]
            )

            res['fieldsets'].append({
                'name': fieldset[0],
                'legend': fieldset[1].get('legend', ''),
                'fields': new_form,
            })
        return res
    return form



from dartcms.utils.loading import get_model
from django.contrib import admin

PageModule = get_model('pages', 'PageModule')
Page = get_model('pages', 'Page')


class PageModuleAdmin(admin.ModelAdmin):
    pass


admin.site.register(PageModule, PageModuleAdmin)


class PageAdmin(admin.ModelAdmin):
    pass


admin.site.register(Page, PageAdmin)



# Generated by Django 1.9.6 on 2016-07-06 06:56
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='pagemodule',
            name='related_model',
            field=models.CharField(default=None, max_length=32, null=True, verbose_name='Related Model'),
        ),
        migrations.AlterField(
            model_name='page',
            name='ad_section',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='pages_page_related', to='ads.AdSection', verbose_name='Ads'),
        ),
    ]



from dartcms.utils.config import DartCMSConfig
from django.utils.translation import ugettext_lazy as _

from .forms import SiteSettingsForm
from .models import SiteSettings

app_name = 'sitesettings'

config = DartCMSConfig({
    'model': SiteSettings,
    'grid': {
        'grid_columns': [
            {'field': 'description', 'width': '20%'},
            {'field': 'type_display', 'width': '20%', 'label': _('Type')},
            {'field': 'value_for_grid', 'width': '60%', 'label': _('Value')},
        ],
        'base_grid_actions': ['update'],
        'model_properties': ['type_display', 'value_for_grid']
    },
    'form': {
        'form_class': SiteSettingsForm,
    }
})

urlpatterns = config.get_urls(exclude=['insert', 'delete'])



# Generated by Django 1.9.8 on 2016-07-28 13:26
from __future__ import unicode_literals

import autoslug.fields
import dartcms.utils.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('feeds', '0003_auto_20160712_1142'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='feed',
            options={'ordering': ['name'], 'verbose_name': 'Feed', 'verbose_name_plural': 'Feeds'},
        ),
        migrations.AlterModelOptions(
            name='feeditem',
            options={'ordering': ['-date_published'], 'verbose_name': 'Feed item', 'verbose_name_plural': 'Feed items'},
        ),
        migrations.AlterModelOptions(
            name='feedtype',
            options={'verbose_name': 'Feed type', 'verbose_name_plural': 'Feed types'},
        ),
        migrations.AddField(
            model_name='feeditem',
            name='is_main',
            field=models.BooleanField(default=False, verbose_name='Is Main'),
        ),
        migrations.AddField(
            model_name='feeditem',
            name='slug',
            field=autoslug.fields.AutoSlugField(default='', editable=False, populate_from=b'name', unique=True, verbose_name='URL'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='feeditem',
            name='full_text',
            field=dartcms.utils.fields.RteField(verbose_name='Full Text'),
        ),
        migrations.AlterField(
            model_name='feeditem',
            name='short_text',
            field=dartcms.utils.fields.RteField(verbose_name='Short Text'),
        ),
    ]



from dartcms.utils.loading import is_model_registered

from .abstract_models import *

__all__ = [
    'FeedType',
    'Feed'
]

if not is_model_registered('feeds', 'FeedItem'):
    class FeedItem(AbstractFeedItem):
        pass

    __all__.append('FeedItem')



from django.db.models import F, Max


def pre_save_handler_section(sender, **kwargs):
    instance = kwargs.get('instance')
    if instance:
        current_level = sender.objects.filter(parent=instance.parent)

        if not current_level.exists():
            instance.sort = 1

        if not instance.id or instance.sort == 0:
            max_sort = current_level.aggregate(Max('sort'))
            s = max_sort.get('sort_max')
            instance.sort = s + 1 if s else 1
        else:
            page_cache = sender.objects.get(pk=instance.id)

            if page_cache.parent == instance.parent:
                if int(page_cache.sort) < int(instance.sort):
                    instance.sort -= 1
                    current_level.filter(sort__gt=page_cache.sort, sort__lte=instance.sort).update(
                        sort=F('sort') - 1)
                else:
                    current_level.filter(sort__gte=instance.sort, sort__lt=page_cache.sort).update(
                        sort=F('sort') + 1)
            else:
                current_level.filter(sort__gte=instance.sort).update(sort=F('sort') + 1)
                sender.objects.filter(parent=page_cache.parent, sort__gt=page_cache.sort).update(sort=F('sort') - 1)


def post_save_handler_section(**kwargs):
    instance = kwargs.get('instance')
    if instance:
        children = instance.children.all()
        if children:
            for child in instance.children.all():
                child.save()


def pre_delete_handler_section(sender, **kwargs):
    instance = kwargs.get('instance')
    if instance:
        sender.objects.filter(parent=instance.parent, sort__gt=instance.sort).update(sort=F('sort') - 1)



from dartcms.apps.users.models import UserGroup
from dartcms.utils.fields import RteField
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import get_language
from mptt.models import MPTTModel, TreeForeignKey


class PageModule(models.Model):
    class Meta:
        app_label = 'pages'
        ordering = ['name']
        verbose_name_plural = _('modules')
        verbose_name = _('module')

    def __str__(self):
        return self.name

    slug = models.SlugField(unique=True, verbose_name=_('Slug'))
    name = models.CharField(max_length=64, verbose_name=_('Name'))
    related_model = models.CharField(verbose_name=_('Related Model'), null=True, default=None, max_length=32,
                                     blank=True)
    is_enabled = models.BooleanField(default=True, verbose_name=_('Is Enabled'))


SECURITY_TYPE_CHOICES = (
    ('DEFAULT', _('Editable for all users')),
    ('BY_PARENT', _('Based on the parent page')),
    ('GROUPS_ONLY', _('Editable for specific groups')),
)


class AbstractPage(MPTTModel):
    """
    Base page model. In common cases, you should not re-declare this class, since it have all necessary features.
    It holds base information about site's pages - urls, names, functional features, parents and childrens etc.
    """

    class Meta:
        app_label = 'pages'
        abstract = True
        ordering = ['sort']
        verbose_name = _('page')
        verbose_name_plural = _('pages')
        unique_together = ['module', 'slug']

    def __str__(self):
        return self.title

    @property
    def full_path(self):
        """
        Constructs breadcrumbs-like page name.
        """
        # todo: There is need to use mptt get_ancestors.
        parent = self.parent
        page_names = [str(self.title)]
        while parent:
            page_names.append(parent.title)
            parent = parent.parent
        return ' / '.join(reversed(page_names))

    parent = TreeForeignKey('self', null=True, related_name='children', verbose_name=_('Parent Page'), blank=True,
                            on_delete=models.CASCADE)

    title = models.CharField(max_length=255, verbose_name=_('Title'), help_text=_('Shows inside the title tag'))
    header = models.CharField(max_length=255, verbose_name=_('Page Header'))

    menu_name = models.CharField(max_length=255, default='', verbose_name=_('Menu name'))
    redirect_url = models.CharField(max_length=255, blank=True, default='', verbose_name=_('URL for Redirect'))

    slug = models.SlugField(default='', verbose_name=_('URL'), blank=True)
    url = models.CharField(max_length=512, unique=True)

    sort = models.IntegerField(default=1)

    module = models.ForeignKey('pages.PageModule', verbose_name=_('Module'),
                               related_name='%(app_label)s_%(class)s_related', on_delete=models.PROTECT)
    module_params = models.CharField(max_length=128, blank=True, null=True, default=None,
                                     verbose_name=_('Module parameters'))

    before_content = RteField(default='', blank=True, verbose_name=_('Before Content'))
    after_content = RteField(default='', blank=True, verbose_name=_('After Content'))

    seo_keywords = models.TextField(default='', blank=True, verbose_name=_('Keywords (meta keywords)'))
    seo_description = models.TextField(default='', blank=True, verbose_name=_('Description (meta description)'))

    ad_section = models.ForeignKey('ads.AdSection', verbose_name=_('Ads'), null=True, blank=True,
                                   related_name='%(app_label)s_%(class)s_related', on_delete=models.SET_NULL)

    security_type = models.CharField(max_length=16, verbose_name=_('Security type'), choices=SECURITY_TYPE_CHOICES,
                                     default='BY_PARENT')
    user_groups = models.ManyToManyField(UserGroup, verbose_name=_('User groups'))

    is_enabled = models.BooleanField(default=True, verbose_name=_('Is Enabled'))
    is_in_menu = models.BooleanField(default=True, verbose_name=_('Show in Menu'))

    date_created = models.DateTimeField(auto_now_add=True)

    def get_security_type(self):
        if self.security_type == 'BY_PARENT' and self.parent:
            parent = self.parent
            while parent:
                if parent.security_type == 'BY_PARENT':
                    parent = parent.parent
                else:
                    break
            return parent.security_type
        else:
            return self.security_type

    def delete(self):
        if not self.parent:
            pass
        else:
            return super(AbstractPage, self).delete()

    @property
    def menu_url(self):
        lang = get_language()
        prefix = '' if lang == settings.LANGUAGE_CODE else '/%s' % lang
        return self.redirect_url if self.redirect_url else prefix + self.url

    @property
    def page_url(self):
        if not self.parent:
            return '/'
        return '/%s/%s/' % (self.module.slug, self.slug)

    @property
    def serializable_object(self):
        obj = {
            'pk': self.pk,
            'parent_id': self.parent_id,
            'title': self.title,
            'module': str(self.module),
            'url': self.url,
        }
        if not self.is_leaf_node():
            obj['state'] = 'closed'
        return obj

    @property
    def serializable_object_with_children(self):
        obj = {
            'pk': self.pk,
            'parent_id': self.parent_id,
            'title': self.title,
            'module': str(self.module),
            'url': self.url,
            'children': []
        }
        for child in self.get_children():
            obj['children'].append(child.serializable_object_with_children)
        return obj



from dartcms.apps.modules.models import Module
from django import forms
from django.utils.translation import ugettext_lazy as _

from .models import CMSUser, UserGroup


class UserForm(forms.ModelForm):
    class Meta:
        model = CMSUser
        exclude = ('password', 'last_login', 'date_joined', 'user_permissions', 'groups', 'is_staff')

    modules = forms.ModelMultipleChoiceField(required=False, label=_('Modules'),
                                             queryset=Module.objects.all())
    user_groups = forms.ModelMultipleChoiceField(required=False, label=_('Groups'),
                                                 queryset=UserGroup.objects.all())



from dartcms.utils.loading import get_model, is_model_registered

DARTCMS_REQUIRED_APPS = [
    'django_gravatar',
    'mptt',

    'dartcms',
    'dartcms.apps.filemanager',
    'dartcms.apps.modules',
    'dartcms.apps.users',
    'dartcms.apps.siteusers',
]

DARTCMS_OPTIONAL_APPS = [
    'dartcms.apps.sitesettings',
    'dartcms.apps.pages',
    'dartcms.apps.ads',
    'dartcms.apps.feedback',
    'dartcms.apps.feeds',
    'dartcms.apps.shop',
]

DARTCMS_APPS = DARTCMS_REQUIRED_APPS + DARTCMS_OPTIONAL_APPS


def get_dartcms_core_apps(include_apps='*', replacements=None):
    """
    If you want to override default DartCMS app, you should pass your app labels (as iterable) to this function.

    Usage:

    INSTALLED_APPS = [
        ...
    ] + get_dartcms_core_apps(replacements=['your_project.pages'])

    This will override default DartCMS app named 'pages'.
    """

    if include_apps == '*' and not replacements:
        return DARTCMS_APPS

    def get_app_label(label):
        pattern = label.replace('dartcms.apps.', '')

        if isinstance(include_apps, (list, tuple)):
            if pattern not in include_apps:
                return

        if replacements:
            for replacement in replacements:
                if replacement.endswith(pattern):
                    return replacement
        return label

    apps = DARTCMS_REQUIRED_APPS
    for app_label in DARTCMS_OPTIONAL_APPS:
        app_label = get_app_label(app_label)
        if app_label:
            apps.append(app_label)

    return apps



from django.db import models
from django.db.models.signals import pre_delete
from django.dispatch.dispatcher import receiver
from django.utils.translation import ugettext_lazy as _


class Folder(models.Model):
    name = models.CharField(max_length=255, unique=True,
                            error_messages={'unique': _('Folder with this name is already exists')})
    date_created = models.DateTimeField(auto_now_add=True)


class File(models.Model):
    folder = models.ForeignKey(Folder, related_name='files', on_delete=models.CASCADE)
    path = models.FileField(upload_to='uploads/%Y/%m/%d', max_length=300)
    date_created = models.DateTimeField(auto_now_add=True)


@receiver(pre_delete, sender=File)
def files_delete(sender, instance, **kwargs):
    instance.path.delete(False)



from dartcms.apps.users.models import UserGroup
from dartcms.utils.config import DartCMSConfig
from django.forms.models import modelform_factory

app_name = 'groups'

config = DartCMSConfig({
    'model': UserGroup,
    'grid': {
        'grid_columns': [
            {'field': 'name', 'width': '100%'},
        ]
    },
    'form': {
        'form_class': modelform_factory(model=UserGroup, exclude=['users'])
    }
})

urlpatterns = config.get_urls()



from dartcms.utils.config import DartCMSConfig
from dartcms.utils.loading import get_model
from django.conf.urls import url
from django.views.decorators.csrf import csrf_exempt
from django.utils.translation import ugettext_lazy as _

from .forms import PageForm
from .views import (AppendPageView, DeletePageView, GetTreeView,
                    InsertPageView, LoadModuleParamsView, MovePageView,
                    UpdatePageView, PageTreeView, OpenUrlRedirectView)

app_name = 'pages'

Page = get_model('pages', 'Page')

config = DartCMSConfig({
    'model': Page,
    'grid': {
        'grid_columns': [
            {'field': 'title', 'width': '60%'},
            {'field': 'url', 'width': '20%'},
            {'field': 'module', 'width': '20%'},
        ],
        'template_name': 'dartcms/apps/pages/grid.html',
        'search': [
            'title', 'url', 'module'
        ],
        'additional_grid_actions': [
            {'url': 'open_url', 'kwarg_name': 'pk', 'label': _('Open URL'), 'icon': 'chevron', 'target': '_blank'}
        ]
    },
    'form': {
        'form_class': PageForm,
        'tabs': True
    }
})

insert_kwargs = config.form.copy()
insert_kwargs['template_name'] = 'dartcms/apps/pages/insert.html'

update_kwargs = config.form.copy()
update_kwargs['template_name'] = 'dartcms/apps/pages/update.html'

delete_kwargs = config.base.copy()
delete_kwargs['template_name'] = 'dartcms/apps/pages/delete.html'

urlpatterns = [
    url(r'^$', PageTreeView.as_view(**config.grid), name='index'),
    url(r'^open_url/(?P<pk>\d+)/$', OpenUrlRedirectView.as_view(), name='open_url'),
    url(r'^get-tree/$', csrf_exempt(GetTreeView.as_view()), name='get_tree'),
    url(r'^insert/$', InsertPageView.as_view(**insert_kwargs), name='insert'),
    url(r'^append/$', csrf_exempt(AppendPageView.as_view()), name='append'),
    url(r'^move/$', csrf_exempt(MovePageView.as_view()), name='move'),
    url(r'^load-module-params/$', LoadModuleParamsView.as_view(), name='load_module_params'),
    url(r'^update/(?P<pk>\d+)/$', UpdatePageView.as_view(**update_kwargs), name='update'),
    url(r'^delete/(?P<pk>\d+)/$', DeletePageView.as_view(**delete_kwargs), name='delete'),
]



# Generated by Django 2.0.1 on 2018-01-30 12:17

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('sitesettings', '0003_auto_20170829_0515'),
    ]

    operations = [
        migrations.AddField(
            model_name='sitesettings',
            name='boolean_value',
            field=models.BooleanField(default=False, verbose_name='Value for boolean'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='object_id',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='sitesettings',
            name='type',
            field=models.CharField(blank=True, choices=[('text', 'String'), ('textarea', 'Text'), ('rich', 'Rich Editor'), ('select', 'Select'), ('date', 'Date'), ('file', 'File'), ('object', 'Object'), ('boolean', 'Checkbox')], default='text', max_length=10, verbose_name='Type'),
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class FileManagerConfig(AppConfig):
    name = 'dartcms.apps.filemanager'
    verbose_name = _('FileManager')



# Generated by Django 1.10.4 on 2016-12-14 03:36
from __future__ import unicode_literals

from dartcms.utils.loading import get_model
from django.db import migrations


def set_homepage_security(apps, schema):
    Page = get_model('pages', 'Page')
    Page.objects.filter(module__slug='homepage').update(security_type='DEFAULT')


class Migration(migrations.Migration):
    dependencies = [
        ('pages', '0011_insert_homepage'),
    ]

    operations = [
        migrations.RunPython(set_homepage_security)
    ]



from dartcms.utils.loading import is_model_registered

from .abstract_models import *

__all__ = [
    'AdPlace',
    'AdSection'
]

if not is_model_registered('ads', 'Ad'):
    class Ad(AbstractAd):
        pass

    __all__.append('Ad')



from dartcms.apps.auth.utils import get_user_model
from dartcms.apps.modules.models import Module, ModuleGroup, ModulePermission
from dartcms.views import AdminMixin, InsertObjectView, UpdateObjectView
from django.contrib.auth import update_session_auth_hash
from django.contrib.auth.forms import AdminPasswordChangeForm
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from django.views.generic import FormView


class ChangePasswordView(AdminMixin, FormView):
    form_class = AdminPasswordChangeForm
    page_header = _('Users')
    template_name = 'dartcms/apps/users/change_password.html'
    success_url = reverse_lazy('dartcms:users:index')

    @property
    def user_model(self):
        return get_user_model()

    def get_form_kwargs(self):
        kwargs = super(ChangePasswordView, self).get_form_kwargs()
        kwargs['user'] = self.user_model.objects.get(pk=self.kwargs['pk'])
        return kwargs

    def get_context_data(self, *args, **kwargs):
        context = super(ChangePasswordView, self).get_context_data(**kwargs)
        context['index_url'] = self.success_url
        return context

    def form_valid(self, form):
        form.save()
        update_session_auth_hash(self.request, form.user)
        return super(ChangePasswordView, self).form_valid(form)


class SaveUserMixin(object):
    def set_module_permissions(self, user):
        read = self.request.POST.getlist('read')
        insert = self.request.POST.getlist('insert')
        update = self.request.POST.getlist('update')
        delete = self.request.POST.getlist('delete')

        for module_id in read:
            user.module_set.add(Module.objects.get(pk=module_id))
            kwargs = {
                'can_insert': module_id in insert,
                'can_update': module_id in update,
                'can_delete': module_id in delete,
                'user_id': user.id,
                'module_id': module_id
            }
            ModulePermission.objects.create(**kwargs)

    def form_valid(self, form):
        data = form.cleaned_data
        user = form.save()
        if user.id:
            user.module_set.clear()
            user.user_module_permissions.all().delete()
            user.user_groups.clear()

        for group in data.get('user_groups', []):
            user.user_groups.add(group)

        self.set_module_permissions(user)

        return self.render_to_json_response({'result': True, 'action': self.action})


class ModulesContextMixin(object):
    def get_context_data(self, **kwargs):
        context = super(ModulesContextMixin, self).get_context_data(**kwargs)
        context['module_groups'] = ModuleGroup.objects.all()
        return context


class CMSUserUpdateView(SaveUserMixin, ModulesContextMixin, UpdateObjectView):
    action = 'UPDATE'
    success_url = reverse_lazy('dartcms:users:index')
    template_name = 'dartcms/apps/users/update.html'

    def get_initial(self):
        obj = self.get_object()
        return {
            'user_groups': obj.user_groups.all()
        }


class CMSUserInsertView(SaveUserMixin, ModulesContextMixin, InsertObjectView):
    template_name = 'dartcms/apps/users/insert.html'
    action = 'INSERT'

    def get_success_url(self):
        return reverse_lazy('dartcms:users:change_password', kwargs={'pk': self.object.pk})



from __future__ import unicode_literals

from django.db import migrations
from django.utils import translation
from django.utils.translation import gettext_lazy as _


def insert_modules(apps, schema):
    from django.conf import settings
    translation.activate(settings.LANGUAGE_CODE)

    ModuleGroup = apps.get_model('modules', 'ModuleGroup')
    Module = apps.get_model('modules', 'Module')

    module = {
        'sort': 4,
        'is_enabled': True,
        'name': _('Orders'),
        'slug': 'shop-orders',
        'description': '',
        'group': ModuleGroup.objects.get(slug='shop')
    }

    Module.objects.create(**module)

    translation.deactivate()


def delete_modules(apps, schema):
    Module = apps.get_model('modules', 'Module')

    Module.objects.get(slug='shop-order').delete()


class Migration(migrations.Migration):
    dependencies = [
        ('modules', '0006_insert_shop_modules'),
    ]

    operations = [
        migrations.RunPython(insert_modules, delete_modules)
    ]



# Generated by Django 1.9.8 on 2016-08-01 16:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('feeds', '0004_auto_20160728_1326'),
    ]

    operations = [
        migrations.AlterField(
            model_name='feeditem',
            name='is_main',
            field=models.BooleanField(default=True, verbose_name='Is Main'),
        ),
    ]



from dartcms.apps.auth.utils import get_user_model
from django.utils.translation import ugettext_lazy as _

from .managers import SiteUserManager

User = get_user_model()


class SiteUser(User):
    class Meta:
        verbose_name = _('Site User')
        verbose_name_plural = _('Site Users')
        proxy = True

    def save(self, *args, **kwargs):
        self.is_staff = False
        super(SiteUser, self).save(*args, **kwargs)

    objects = SiteUserManager()



# Generated by Django 3.2.8 on 2021-10-08 12:46

from django.db import migrations


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('user', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SiteUser',
            fields=[
            ],
            options={
                'verbose_name': 'Site User',
                'verbose_name_plural': 'Site Users',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('user.user',),
        ),
    ]



from dartcms.utils.loading import get_model, is_model_registered
from django.db.models.signals import post_save, pre_delete, pre_save

from .abstract_models import *
from .signals import *

__all__ = [
    'PageModule'
]

if is_model_registered('pages', 'Page'):
    page_model = get_model('pages', 'Page')
else:
    class Page(AbstractPage):
        pass

    __all__.append('Page')

    page_model = Page

pre_save.connect(pre_save_handler, sender=page_model)
post_save.connect(post_save_handler, sender=page_model)
pre_delete.connect(pre_delete_handler, sender=page_model)



# Generated by Django 2.0.1 on 2018-02-14 13:42

import versatileimagefield.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('shop', '0009_auto_20180123_1113'),
    ]

    operations = [
        migrations.AlterField(
            model_name='product',
            name='image',
            field=versatileimagefield.fields.VersatileImageField(blank=True, null=True, upload_to='shop/product/%Y/%m/%d', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='productcatalog',
            name='image',
            field=versatileimagefield.fields.VersatileImageField(blank=True, null=True, upload_to='shop/catalog', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='productimage',
            name='image',
            field=versatileimagefield.fields.VersatileImageField(upload_to='shop/product_images/%Y/%m/%d', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='productlabel',
            name='image',
            field=versatileimagefield.fields.VersatileImageField(blank=True, null=True, upload_to='shop/label', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='productmanufacturer',
            name='image',
            field=versatileimagefield.fields.VersatileImageField(blank=True, null=True, upload_to='shop/producer', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='productsection',
            name='image',
            field=versatileimagefield.fields.VersatileImageField(blank=True, null=True, upload_to='shop/section', verbose_name='Image'),
        ),
    ]



# Generated by Django 1.9.6 on 2016-07-06 06:54
from __future__ import unicode_literals

from django.db import migrations
from django.utils import translation
from django.utils.translation import gettext_lazy as _


def insert_module(apps, schema):
    from django.conf import settings
    translation.activate(settings.LANGUAGE_CODE)

    PageModule = apps.get_model('pages', 'PageModule')

    PageModule.objects.create(slug='feedback', name=_('Feedback'), related_model='feedback.FormType'),

    translation.deactivate()


def drop_module(apps, schema):
    PageModule = apps.get_model('pages', 'PageModule')

    PageModule.objects.get(slug='feedback').delete()


class Migration(migrations.Migration):
    dependencies = [
        ('pages', '0006_auto_20160711_1627'),
    ]

    operations = [
        migrations.RunPython(insert_module, drop_module)
    ]



from dartcms.utils.config import DartCMSConfig

from .models import FormType

app_name = 'feedback'

config = DartCMSConfig({
    'model': FormType,
    'grid': {
        'grid_columns': [
            {'field': 'name', 'width': '70%'},
            {'field': 'slug', 'width': '30%'},
        ],
        'additional_grid_actions': [
            {'url': 'messages', 'kwarg_name': 'form_type', 'include_urls': 'dartcms.apps.feedback.messages.urls'}
        ]
    },
})

urlpatterns = config.get_urls()



from __future__ import unicode_literals

from django.db import migrations
from django.utils import translation
from django.utils.translation import gettext_lazy as _

MODULE_GROUPS = [
    {
        'sort': 5,
        'description': '',
        'fa': 'fa-shopping-cart',
        'slug': 'shop',
        'name': _('Shop'),
        'modules': [
            {
                'sort': 1,
                'is_enabled': True,
                'name': _('Product catalogs'),
                'slug': 'shop-catalog',
                'description': ''
            },
            {
                'sort': 2,
                'is_enabled': True,
                'name': _('Product manufacturers'),
                'slug': 'shop-manufactures',
                'description': ''
            },
            {
                'sort': 3,
                'is_enabled': True,
                'name': _('Product labels'),
                'slug': 'shop-labels',
                'description': ''
            },
        ]
    },
]


def insert_modules(apps, schema):
    from django.conf import settings
    translation.activate(settings.LANGUAGE_CODE)

    ModuleGroup = apps.get_model('modules', 'ModuleGroup')
    Module = apps.get_model('modules', 'Module')

    for group in MODULE_GROUPS:
        group_modules = group.pop('modules', [])
        group = ModuleGroup.objects.create(**group)
        for module in group_modules:
            module['group'] = group
            Module.objects.create(**module)

    translation.deactivate()


def delete_modules(apps, schema):
    ModuleGroup = apps.get_model('modules', 'ModuleGroup')

    for group in MODULE_GROUPS:
        ModuleGroup.objects.get(slug=group['slug']).delete()


class Migration(migrations.Migration):
    dependencies = [
        ('modules', '0005_insert_modules'),
    ]

    operations = [
        migrations.RunPython(insert_modules, delete_modules)
    ]



from dartcms import get_model
from dartcms.utils.config import DartCMSConfig
from django.utils.translation import ugettext_lazy as _

from .forms import ProductCatalogForm

app_name = 'catalog'

ProductCatalog = get_model('shop', 'ProductCatalog')

config = DartCMSConfig({
    'model': ProductCatalog,
    'grid': {
        'grid_columns': [
            {'field': 'name', 'width': '50%'},
            {'field': 'slug', 'width': '40%'},
            {'field': 'is_visible', 'width': '10%'},
        ],
        'additional_grid_actions': [
            {'url': 'sections', 'label': _('Sections'),
             'kwarg_name': 'catalog', 'include_urls': 'dartcms.apps.shop.section.urls'}
        ]
    },
    'form': {
        'form_class': ProductCatalogForm
    }
})

urlpatterns = config.get_urls()



# Generated by Django 2.0.1 on 2018-01-23 11:13

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0009_alter_user_last_name_max_length'),
        ('users', '0003_user_group'),
    ]

    operations = [
        migrations.CreateModel(
            name='CMSUser',
            fields=[
            ],
            options={
                'verbose_name': 'CMS User',
                'verbose_name_plural': 'CMS Users',
                'proxy': True,
                'indexes': [],
            },
            bases=(settings.AUTH_USER_MODEL,),
        ),
    ]



from dartcms.apps.auth.mixins import StaffRequiredMixin
from django.views.generic import TemplateView


class DashboardIndexView(StaffRequiredMixin, TemplateView):
    template_name = 'dartcms/apps/dashboard/index.html'



# Generated by Django 1.9.7 on 2016-07-11 16:27
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0005_pagemodules_relationships'),
    ]

    operations = [
        migrations.AlterField(
            model_name='page',
            name='url',
            field=models.CharField(max_length=512, unique=True),
        ),
    ]



from dartcms.utils.loading import get_model
from dartcms.views import (DeleteObjectView, GridView, InsertObjectView,
                           UpdateObjectView)
from django.forms import modelform_factory

MESSAGES_MODELS_MAPPING = {
    'contact': get_model('feedback', 'ContactMessage'),
    'question': get_model('feedback', 'QuestionMessage'),
}


class DynamicModelMixin(object):
    def setup_model(self):
        form_type = get_model('feedback', 'FormType').objects.get(pk=self.kwargs['form_type'])
        self.model = MESSAGES_MODELS_MAPPING.get(form_type.slug)

    def get_queryset(self):
        self.setup_model()
        return super(DynamicModelMixin, self).get_queryset()


class FeedbackModelMixin(DynamicModelMixin):
    def get_form_class(self):
        self.setup_model()
        return modelform_factory(self.model, exclude=['type'])


class MessagesGridView(DynamicModelMixin, GridView):
    pass


class UpdateMessageFormView(FeedbackModelMixin, UpdateObjectView):
    pass


class DeleteMessageFormView(FeedbackModelMixin, DeleteObjectView):
    pass


class InsertMessageFormView(FeedbackModelMixin, InsertObjectView):
    pass



import sys
import io

from django.core.serializers.json import Serializer
from django.db import models
from django.utils.encoding import force_text, is_protected_type

if sys.version_info.major == 3:
    unicode = str


class DartCMSSerializer(Serializer):
    """
    Jquery EasyUI format's compatible serializer for django db objects.
    """

    def serialize(self, queryset, **options):
        self.options = options
        self.stream = options.pop('stream', io.StringIO())
        self.selected_fields = options.pop('fields', None)
        self.selected_props = options.pop('props', [])
        self.use_natural_keys = options.pop('use_natural_keys', False)
        self.use_natural_foreign_keys = options.pop('use_natural_foreign_keys', False)
        self.use_natural_primary_keys = options.pop('use_natural_primary_keys', False)

        self.start_serialization()
        self.first = True
        for obj in queryset:
            self.start_object(obj)
            concrete_model = obj._meta.concrete_model
            for field in concrete_model._meta.local_fields:
                if field.serialize:
                    if field.remote_field is None:
                        if self.selected_fields is None or field.attname in self.selected_fields:
                            self.handle_field(obj, field)
                    else:
                        if self.selected_fields is None or field.attname[:-3] in self.selected_fields:
                            self.handle_fk_field(obj, field)
            for field in concrete_model._meta.many_to_many:
                if field.serialize:
                    if self.selected_fields is None or field.attname in self.selected_fields:
                        self.handle_m2m_field(obj, field)
            if self.selected_props:
                for field in self.selected_props:
                    self.handle_prop(obj, field)
            self.end_object(obj)
            if self.first:
                self.first = False
        self.end_serialization()
        return self.getvalue()

    def handle_prop(self, obj, field):
        self._current[field] = getattr(obj, field)
        if callable(self._current[field]):
            self._current[field] = self._current[field]()

    def get_dump_object(self, obj):
        data = self._current
        if not self.use_natural_primary_keys or not hasattr(obj, 'natural_key'):
            data['pk'] = force_text(obj._get_pk_val(), strings_only=True)
        return data

    def handle_field(self, obj, field):
        if isinstance(field, (models.ImageField, models.FileField)):
            value = field.value_from_object(obj)
            if value:
                self._current[field.name] = value.url
            else:
                self._current[field.name] = None
        else:
            value = field.value_from_object(obj)
            if is_protected_type(value):
                self._current[field.name] = value
            else:
                self._current[field.name] = field.value_to_string(obj)

    def handle_fk_field(self, obj, field):
        if self.use_natural_foreign_keys and hasattr(field.remote_field.model, 'natural_key'):
            related = getattr(obj, field.name)
            if related:
                value = related.natural_key()
            else:
                value = None
        else:
            value = unicode(getattr(obj, field.name))
        self._current[field.name] = value



# Generated by Django 1.10.4 on 2017-08-29 05:15
from __future__ import unicode_literals

import autoslug.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('feeds', '0005_auto_20160801_1607'),
    ]

    operations = [
        migrations.AlterField(
            model_name='feeditem',
            name='picture',
            field=models.ImageField(blank=True, null=True, upload_to='feeds/%Y/%m/%d', verbose_name='Picture'),
        ),
        migrations.AlterField(
            model_name='feeditem',
            name='slug',
            field=autoslug.fields.AutoSlugField(editable=False, populate_from='name', unique=True, verbose_name='URL'),
        ),
    ]



# Generated by Django 1.9.7 on 2016-07-29 02:29
from __future__ import unicode_literals

import jsonfield.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('modules', '0007_insert_shop_modules'),
    ]

    operations = [
        migrations.AddField(
            model_name='module',
            name='config',
            field=jsonfield.fields.JSONField(blank=True, null=True, verbose_name='Config'),
        ),
    ]



# Generated by Django 1.10.4 on 2017-08-29 05:15
from __future__ import unicode_literals

import autoslug.fields
import dartcms.utils.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shop', '0007_runtime_config'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='order',
            options={'ordering': ['-date_created'], 'verbose_name': 'order', 'verbose_name_plural': 'orders'},
        ),
        migrations.AlterModelOptions(
            name='orderdetail',
            options={'ordering': ['name'], 'verbose_name': 'order datail', 'verbose_name_plural': 'order datails'},
        ),
        migrations.AlterModelManagers(
            name='productsection',
            managers=[
            ],
        ),
        migrations.RemoveField(
            model_name='order',
            name='user',
        ),
        migrations.AlterField(
            model_name='order',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Date created'),
        ),
        migrations.AlterField(
            model_name='orderpaymenttype',
            name='description',
            field=dartcms.utils.fields.RteField(blank=True, default='', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='ordershippingtype',
            name='description',
            field=dartcms.utils.fields.RteField(blank=True, default='', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='orderstatus',
            name='description',
            field=dartcms.utils.fields.RteField(blank=True, default='', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='product',
            name='code',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='Code'),
        ),
        migrations.AlterField(
            model_name='product',
            name='description',
            field=dartcms.utils.fields.RteField(blank=True, default='', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='product',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to='shop/product/%Y/%m/%d', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='product',
            name='seo_description',
            field=models.TextField(blank=True, default='', verbose_name='Description (meta description)'),
        ),
        migrations.AlterField(
            model_name='product',
            name='seo_keywords',
            field=models.TextField(blank=True, default='', verbose_name='Keywords (meta keywords)'),
        ),
        migrations.AlterField(
            model_name='product',
            name='short_description',
            field=dartcms.utils.fields.RteField(blank=True, default='', verbose_name='Short description'),
        ),
        migrations.AlterField(
            model_name='product',
            name='slug',
            field=autoslug.fields.AutoSlugField(editable=False, populate_from='name', unique=True, verbose_name='URL'),
        ),
        migrations.AlterField(
            model_name='productcatalog',
            name='description',
            field=dartcms.utils.fields.RteField(blank=True, default='', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='productcatalog',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to='shop/catalog', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='productcatalog',
            name='seo_description',
            field=models.TextField(blank=True, default='', verbose_name='Description (meta description)'),
        ),
        migrations.AlterField(
            model_name='productcatalog',
            name='seo_keywords',
            field=models.TextField(blank=True, default='', verbose_name='Keywords (meta keywords)'),
        ),
        migrations.AlterField(
            model_name='productimage',
            name='image',
            field=models.ImageField(upload_to='shop/product_images/%Y/%m/%d', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='productlabel',
            name='description',
            field=dartcms.utils.fields.RteField(blank=True, default='', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='productlabel',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to='shop/label', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='productlabel',
            name='seo_description',
            field=models.TextField(blank=True, default='', verbose_name='Description (meta description)'),
        ),
        migrations.AlterField(
            model_name='productlabel',
            name='seo_keywords',
            field=models.TextField(blank=True, default='', verbose_name='Keywords (meta keywords)'),
        ),
        migrations.AlterField(
            model_name='productmanufacturer',
            name='description',
            field=dartcms.utils.fields.RteField(blank=True, default='', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='productmanufacturer',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to='shop/producer', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='productmanufacturer',
            name='seo_description',
            field=models.TextField(blank=True, default='', verbose_name='Description (meta description)'),
        ),
        migrations.AlterField(
            model_name='productmanufacturer',
            name='seo_keywords',
            field=models.TextField(blank=True, default='', verbose_name='Keywords (meta keywords)'),
        ),
        migrations.AlterField(
            model_name='productmanufacturer',
            name='slug',
            field=autoslug.fields.AutoSlugField(editable=False, populate_from='name', unique=True, verbose_name='URL'),
        ),
        migrations.AlterField(
            model_name='productsection',
            name='description',
            field=dartcms.utils.fields.RteField(blank=True, default='', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='productsection',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to='shop/section', verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='productsection',
            name='seo_description',
            field=models.TextField(blank=True, default='', verbose_name='Description (meta description)'),
        ),
        migrations.AlterField(
            model_name='productsection',
            name='seo_keywords',
            field=models.TextField(blank=True, default='', verbose_name='Keywords (meta keywords)'),
        ),
        migrations.AlterField(
            model_name='productsection',
            name='slug',
            field=autoslug.fields.AutoSlugField(editable=False, populate_from='name', unique=True, verbose_name='URL'),
        ),
    ]



from django.db import models


def get_model_field_label(model, field_label):
    """
    Constructs label from model's field.
    """
    label = field_label.title()

    for field in model._meta.fields:
        column_field = field_label
        if isinstance(field, models.ForeignKey):
            column_field += '_id'

        if field.attname == column_field:
            label = getattr(field, 'verbose_name')
            if not label:
                label = field.attname.replace('_', '')
            break

    return label.title()


def get_model_field(model, field_label):
    """
    Returns model's field.
    """
    for field in model._meta.fields:
        column_field = field_label
        if isinstance(field, models.ForeignKey):
            column_field += '_id'

        if field.attname == column_field:
            return field


FIELD_TYPES_MAPPING = {
    models.DateTimeField: 'DATETIME',
    models.DateField: 'DATE',
    models.TimeField: 'TIME',
    models.BooleanField: 'BOOLEAN',
    models.ForeignKey: 'FOREIGN_KEY',
    models.ImageField: 'IMAGE',
    models.FileField: 'FILE'
}


def get_model_field_type(model, field_label):
    """
    Returns model's field type.
    """
    return FIELD_TYPES_MAPPING.get(type(get_model_field(model, field_label)), 'STRING')



from __future__ import unicode_literals

from django.db import migrations
from django.utils import translation
from django.utils.translation import gettext_lazy as _

records = [
    dict(sort=1, slug='accepted', name=_('Accepted')),
    dict(sort=2, slug='performed', name=_('Performed')),
    dict(sort=3, slug='awaiting_payment', name=_('Avaiting payment')),
    dict(sort=4, slug='delivered', name=_('Delivered')),
    dict(sort=5, slug='ready', name=_('Ready')),
    dict(sort=6, slug='completed', name=_('Completed')),
    dict(sort=7, slug='canceled', name=_('Canceled'))
]


def insert(apps, schema):
    from django.conf import settings
    translation.activate(settings.LANGUAGE_CODE)

    model = apps.get_model('shop', 'OrderStatus')
    for record in records:
        model.objects.create(**record)

    translation.deactivate()


def delete(apps, schema):
    model = apps.get_model('shop', 'OrderStatus')
    for record in records:
        model.objects.get(slug=record['slug']).delete()


class Migration(migrations.Migration):
    dependencies = [
        ('shop', '0005_insert_shipping_types'),
    ]

    operations = [
        migrations.RunPython(insert, delete)
    ]



# Generated by Django 2.0.1 on 2018-01-23 11:13

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0013_auto_20170829_0515'),
    ]

    operations = [
        migrations.AlterField(
            model_name='page',
            name='ad_section',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='pages_page_related', to='ads.AdSection', verbose_name='Ads'),
        ),
        migrations.AlterField(
            model_name='page',
            name='module',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='pages_page_related', to='pages.PageModule', verbose_name='Module'),
        ),
    ]



from django import template
from django.template import Node, Variable, NodeList, VariableDoesNotExist


def do_startswith(parser, token, negate):
    try:
        tag_name, string, start_string = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError("%r tag requires two arguments" % token.contents.split()[0])
    end_tag = 'end' + tag_name
    nodelist_true = parser.parse(('else', end_tag))
    token = parser.next_token()
    if token.contents == 'else':
        nodelist_false = parser.parse((end_tag,))
        parser.delete_first_token()
    else:
        nodelist_false = NodeList()
    return IfStartsWithNode(string, start_string, nodelist_true, nodelist_false, negate)


class IfStartsWithNode(Node):
    def __init__(self, string, start_string, nodelist_true, nodelist_false, negate):
        self.start_string, self.string = Variable(start_string), Variable(string)
        self.nodelist_true, self.nodelist_false = nodelist_true, nodelist_false
        self.negate = negate
        self.negate = negate

    def __repr__(self):
        return "<IfStartsWithNode>"

    def render(self, context):
        try:
            string = self.string.resolve(context)
        except VariableDoesNotExist:
            string = None
        try:
            start_string = self.start_string.resolve(context)
        except VariableDoesNotExist:
            start_string = None

        if (self.negate and not string.startswith(start_string)) or (
                not self.negate and string.startswith(start_string)):
            return self.nodelist_true.render(context)
        return self.nodelist_false.render(context)


# Generated by Django 1.9.7 on 2016-07-29 03:45
from __future__ import unicode_literals

from django.db import migrations


def update_runtime_config(apps, schema):
    Module = apps.get_model('modules', 'Module')
    models_mapping = {
        'shop-manufactures': 'shop.ProductManufacturer',
        'shop-labels': 'shop.ProductLabel',
    }
    for module in ('shop-manufactures', 'shop-labels'):
        try:
            m = Module.objects.get(slug=module)
        except Module.DoesNotExist:
            continue

        m.slug = 'dict-%s' % m.slug
        m.config = {
            "model": models_mapping.get(module)
        }
        m.save()


class Migration(migrations.Migration):
    dependencies = [
        ('shop', '0006_insert_order_statuses'),
    ]

    operations = [
        migrations.RunPython(update_runtime_config)
    ]



# Generated by Django 1.9.7 on 2016-07-15 13:54
from __future__ import unicode_literals

import dartcms.utils.fields
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('shop', '0002_auto_20160714_1249'),
    ]

    operations = [
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('fullname', models.CharField(max_length=255, verbose_name='Fullname')),
                ('email', models.EmailField(blank=True, max_length=254, null=True, verbose_name='E-mail')),
                ('shipping_address', models.TextField(blank=True, null=True, verbose_name='Shipping address')),
                ('shipping_cost', models.DecimalField(decimal_places=2, default=0, max_digits=10, verbose_name='Cost')),
                ('is_paid', models.BooleanField(default=False, verbose_name='Is paid')),
                ('paid_sum', models.DecimalField(blank=True, decimal_places=2, max_digits=10, null=True, verbose_name='Paid sum')),
                ('discount_percent', models.DecimalField(decimal_places=2, default=0, max_digits=5, verbose_name='Discount')),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_modified', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ['-date_created'],
                'abstract': False,
                'verbose_name': 'order status',
                'verbose_name_plural': 'order statuses',
            },
        ),
        migrations.CreateModel(
            name='OrderDetail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(blank=True, max_length=255, verbose_name='Code')),
                ('name', models.CharField(max_length=1024, verbose_name='Name')),
                ('price', models.DecimalField(decimal_places=2, default=0, max_digits=10, verbose_name='Price')),
                ('quantity', models.IntegerField(default=0, verbose_name='Quantity')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='details', to='shop.Order', verbose_name='Order')),
            ],
            options={
                'ordering': ['name'],
                'abstract': False,
                'verbose_name': 'order_datil',
                'verbose_name_plural': 'order_datils',
            },
        ),
        migrations.CreateModel(
            name='OrderPaymentType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(unique=True, verbose_name='URL')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('description', dartcms.utils.fields.RteField(blank=True, default=b'', verbose_name='Description')),
                ('is_enabled', models.BooleanField(default=True, verbose_name='Is enabled')),
                ('sort', models.IntegerField(default=1)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ['sort'],
                'abstract': False,
                'verbose_name': 'payment type',
                'verbose_name_plural': 'payment types',
            },
        ),
        migrations.CreateModel(
            name='OrderShippingType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(unique=True, verbose_name='URL')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('description', dartcms.utils.fields.RteField(blank=True, default=b'', verbose_name='Description')),
                ('cost', models.DecimalField(decimal_places=2, default=0, max_digits=10, verbose_name='Cost')),
                ('is_enabled', models.BooleanField(default=True, verbose_name='Is enabled')),
                ('sort', models.IntegerField(default=1)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ['sort'],
                'abstract': False,
                'verbose_name': 'shipping type',
                'verbose_name_plural': 'shipping types',
            },
        ),
        migrations.CreateModel(
            name='OrderStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(unique=True, verbose_name='URL')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('description', dartcms.utils.fields.RteField(blank=True, default=b'', verbose_name='Description')),
                ('is_enabled', models.BooleanField(default=True, verbose_name='Is enabled')),
                ('sort', models.IntegerField(default=1)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ['sort'],
                'abstract': False,
                'verbose_name': 'order status',
                'verbose_name_plural': 'order statuses',
            },
        ),
        migrations.AlterField(
            model_name='product',
            name='manufacturer',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='manufacturer_products', to='shop.ProductManufacturer', verbose_name='Manufacturer'),
        ),
        migrations.AddField(
            model_name='orderdetail',
            name='product',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='shop.Product', verbose_name='Product'),
        ),
        migrations.AddField(
            model_name='order',
            name='payment_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='shop.OrderPaymentType', verbose_name='Payment type'),
        ),
        migrations.AddField(
            model_name='order',
            name='shipping_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='shop.OrderShippingType', verbose_name='Shipping type'),
        ),
        migrations.AddField(
            model_name='order',
            name='status',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='shop.OrderStatus', verbose_name='Status'),
        ),
        migrations.AddField(
            model_name='order',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
    ]



# Generated by Django 2.0.1 on 2018-02-14 13:30

import versatileimagefield.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('feeds', '0007_auto_20180123_1113'),
    ]

    operations = [
        migrations.AlterField(
            model_name='feeditem',
            name='picture',
            field=versatileimagefield.fields.VersatileImageField(blank=True, null=True, upload_to='feeds/%Y/%m/%d', verbose_name='Picture'),
        ),
    ]



import re

from dartcms.apps.modules.functions import (get_current_module,
                                            get_current_module_perms)
from dartcms.utils.loading import get_form_class, get_model
from django.forms import modelform_factory
from django.http import Http404, JsonResponse
from django.urls import reverse
from django.utils.functional import cached_property


class ModulePermissionsMixin(object):
    def check_module_perms(self):
        if not self.request.user.is_authenticated or not self.request.user.is_staff:
            return False

        root = reverse('dartcms:dashboard:index')
        active_module_slug = self.request.path.replace(root, '', 1).strip('/').split('/')[0]
        user_modules = self.request.user.module_set.all().values_list('slug', flat=True)
        return active_module_slug in list(user_modules)

    @cached_property
    def user_module_permissions(self):
        return get_current_module_perms(self.request)

    def dispatch(self, request, *args, **kwargs):
        if not self.check_module_perms():
            raise Http404('You don\'t have permissions to this module')

        response = super(ModulePermissionsMixin, self).dispatch(request, *args, **kwargs)
        return response


class AdminMixin(ModulePermissionsMixin):
    config = None

    module = None
    model = None
    form_class = None

    parent_model = None
    parent_model_fk = None

    parent_kwarg_name = ''

    page_header = ''

    tabs = False

    def dispatch(self, request, *args, **kwargs):
        self.module = get_current_module(request.path)
        if self.module and self.module.config:
            self.config = self.module.config

            model_label = self.config.get('model', self.model)
            if model_label:
                app_label, model_name = model_label.split('.')
                self.model = get_model(app_label, model_name)

            form_config = self.config.get('form')
            if form_config:
                if 'form_class' in form_config:
                    app_label, form_class_name = form_config['form_class'].split('.')
                    self.form_class = get_form_class(app_label, form_class_name)
                form_fields = form_config.get('fields', '__all__')
                exclude_fields = form_config.get('exclude', [])
            else:
                form_fields = '__all__'
                exclude_fields = []

            if self.form_class is None:
                self.form_class = modelform_factory(self.model, fields=form_fields, exclude=exclude_fields)

            if hasattr(self, 'grid_columns'):
                grid_config = self.config.get('grid')
                if grid_config:
                    self.grid_columns = grid_config.get('grid_columns', self.grid_columns)

                    search_config = grid_config.get('search')
                    if search_config:
                        self.search = search_config

                    self.ordering = grid_config.get('ordering')
                    self.model_properties = grid_config.get('model_properties')

        return super(AdminMixin, self).dispatch(request, *args, **kwargs)

    def get_foreign_key_name(self):
        if self.parent_model_fk or self.parent_model:
            return self.parent_model_fk if self.parent_model_fk else '%s_id' % self.parent_model.__name__.lower()

    def get_context_data(self, *args, **kwargs):
        context = super(AdminMixin, self).get_context_data(*args, **kwargs)

        if self.parent_kwarg_name:
            reg = r'(%s/(\d+)/)$' % self.kwargs['children_url']
            parent_url = re.sub(reg, '', self.request.path)
        else:
            parent_url = ''

        context.update({
            'tabs': self.tabs,
            'page_header': self.page_header if self.page_header else self.model._meta.verbose_name_plural,
            'parent_kwarg_name': self.parent_kwarg_name,
            'parent_url': parent_url
        })
        return context


class JSONResponseMixin(object):
    """
    A mixin that can be used to render a JSON response.
    """
    http_method_names = ['get', 'post']

    def render_to_json_response(self, context, **response_kwargs):
        """
        Returns a JSON response, transforming 'context' to make the payload.
        """
        safe = response_kwargs.pop('safe', None) or False
        return JsonResponse(self.get_data(context), safe=safe, **response_kwargs)

    def get_data(self, context):
        """
        Returns an object that will be serialized as JSON by json.dumps().
        """
        return context



import datetime

from autoslug import AutoSlugField
from dartcms.utils.fields import RteField
from django.db import models
from django.utils.translation import ugettext_lazy as _
from versatileimagefield.fields import VersatileImageField


class FeedType(models.Model):
    class Meta:
        app_label = 'feeds'
        verbose_name = _('Feed type')
        verbose_name_plural = _('Feed types')

    def __str__(self):
        return self.name

    name = models.CharField(max_length=255, verbose_name=_('Name'))
    slug = models.SlugField(verbose_name=_('Slug'))


class Feed(models.Model):
    class Meta:
        app_label = 'feeds'
        ordering = ['name']
        verbose_name = _('Feed')
        verbose_name_plural = _('Feeds')

    def __str__(self):
        return self.name

    type = models.ForeignKey(FeedType, verbose_name=_('Type'), related_name='feeds', on_delete=models.PROTECT)
    name = models.CharField(max_length=255, verbose_name=_('Name'))

    def delete(self, **kwargs):
        try:
            return super(Feed, self).delete(**kwargs)
        except models.ProtectedError:
            # todo raise exeption and show it by messages framework
            pass


class AbstractFeedItem(models.Model):
    class Meta:
        app_label = 'feeds'
        ordering = ['-date_published']
        abstract = True
        verbose_name = _('Feed item')
        verbose_name_plural = _('Feed items')

    @property
    def search_content_type(self):
        return self.feed.type.slug

    def __str__(self):
        return self.name

    slug = AutoSlugField(_('URL'), populate_from='name', unique=True)
    feed = models.ForeignKey(Feed, verbose_name=_('Feed'), on_delete=models.PROTECT)
    name = models.CharField(max_length=1024, verbose_name=_('Title'))
    short_text = RteField(verbose_name=_('Short Text'))
    full_text = RteField(verbose_name=_('Full Text'))
    picture = VersatileImageField(verbose_name=_('Picture'), upload_to='feeds/%Y/%m/%d', null=True, blank=True)
    seo_keywords = models.TextField(
        verbose_name=_('Keywords'),
        help_text=_('Do not use more than 255 symbols'),
        blank=True
    )
    seo_description = models.TextField(
        verbose_name=_('Description'),
        help_text=_('Do not use more than 1024 symbols'),
        blank=True
    )
    is_visible = models.BooleanField(default=True, verbose_name=_('Show on Site'))
    is_main = models.BooleanField(default=True, verbose_name=_('Is Main'))
    date_published = models.DateTimeField(default=datetime.datetime.now, verbose_name=_('Date of Publication'))
    date_created = models.DateTimeField(auto_now_add=True, verbose_name=_('Date of Creation'))



# Generated by Django 2.1.3 on 2018-11-14 10:21

from django.db import migrations
import versatileimagefield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('ads', '0002_auto_20170829_0515'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ad',
            name='picture',
            field=versatileimagefield.fields.VersatileImageField(blank=True, null=True, upload_to='b/%Y/%m/%d', verbose_name='Picture'),
        ),
    ]



from dartcms.apps.sitesettings.models import SiteSettings
from django.forms import ModelForm, Select, TextInput

FORM_MAP = {
    SiteSettings.OBJECT: ('object_id', ['text_value', 'file_value', 'boolean_value', 'date_value']),
    SiteSettings.BOOLEAN: ('boolean_value', ['text_value', 'file_value', 'object_id', 'date_value']),
    SiteSettings.DATE: ('date_value', ['text_value', 'file_value', 'object_id', 'boolean_value']),
    SiteSettings.FILE: ('file_value', ['text_value', 'date_value', 'object_id', 'boolean_value']),
    SiteSettings.TEXT: ('text_value', ['file_value', 'date_value', 'object_id', 'boolean_value'])
}


class SiteSettingsForm(ModelForm):
    class Meta:
        model = SiteSettings
        exclude = ['slug', 'description', 'type', 'options', 'content_type']

    def __init__(self, *args, **kwargs):
        super(SiteSettingsForm, self).__init__(*args, **kwargs)

        meta = getattr(self, 'Meta', None)
        exclude = getattr(meta, 'exclude', [])

        form_map = FORM_MAP.get(self.instance.type, FORM_MAP[SiteSettings.TEXT])
        self.fields[form_map[0]].label = self.instance.description
        exclude = exclude + form_map[1]

        if self.instance.type == self.instance.OBJECT:
            options = [(0, '---')] + [
                (choice.pk, choice) for choice in self.instance.content_type.model_class().objects.all()]
            self.fields['object_id'].widget = Select(choices=options)
        elif self.instance.type == self.instance.RICH:
            self.fields['text_value'].widget.attrs = {'class': 'rte'}
        elif self.instance.type == self.instance.TEXT:
            self.fields['text_value'].widget = TextInput()
        elif self.instance.type == self.instance.SELECT:
            options = [(o, o) for o in self.instance.options.split(";")]
            self.fields['text_value'].widget = Select(choices=options)

        for field_name in exclude:
            if field_name in self.fields:
                del self.fields[field_name]



from autoslug import AutoSlugField
from dartcms.utils.fields import RteField
from dartcms.utils.loading import get_model
from django.db import models
from django.db.models import F, Sum
from django.utils.translation import ugettext as __
from django.utils.translation import ugettext_lazy as _
from mptt.models import MPTTModel, TreeForeignKey
from versatileimagefield.fields import VersatileImageField


class AbstractProductBase(models.Model):
    class Meta:
        abstract = True
        app_label = 'shop'

    def __str__(self):
        return self.name

    name = models.CharField(_('Name'), max_length=255)
    description = RteField(_('Description'), blank=True, default='')
    seo_keywords = models.TextField(_('Keywords (meta keywords)'), default='', blank=True)
    seo_description = models.TextField(_('Description (meta description)'), default='', blank=True)
    is_visible = models.BooleanField(_('Is Visible'), default=True)
    date_created = models.DateTimeField(auto_now_add=True)


class AbstractProductCatalog(AbstractProductBase):
    class Meta:
        abstract = True
        app_label = 'shop'
        ordering = ['name']
        verbose_name_plural = _('product catalogs')
        verbose_name = _('product catalog')

    slug = models.SlugField(_('URL'), unique=True)
    image = VersatileImageField(_('Image'), upload_to='shop/catalog', null=True, blank=True)


class AbstractProductManufacturer(AbstractProductBase):
    class Meta:
        abstract = True
        app_label = 'shop'
        ordering = ['name']
        verbose_name_plural = _('product manufacturers')
        verbose_name = _('product manufacturer')

    slug = AutoSlugField(_('URL'), populate_from='name', unique=True)
    image = VersatileImageField(_('Image'), upload_to='shop/producer', null=True, blank=True)


class AbstractProductSection(MPTTModel, AbstractProductBase):
    class Meta:
        abstract = True
        app_label = 'shop'
        ordering = ['sort']
        verbose_name = _('product section')
        verbose_name_plural = _('product sections')
        unique_together = ['catalog', 'slug']

    slug = AutoSlugField(_('URL'), populate_from='name', unique=True)
    image = VersatileImageField(_('Image'), upload_to='shop/section', null=True, blank=True)
    parent = TreeForeignKey('self', null=True, related_name='children', verbose_name=_('Parent Section'), blank=True,
                            on_delete=models.PROTECT)
    catalog = models.ForeignKey('shop.ProductCatalog', verbose_name=_('Product catalog'), related_name='sections',
                                on_delete=models.PROTECT)
    sort = models.IntegerField(default=1)

    def __str__(self):
        parent = self.parent
        section_names = [self.name]
        while parent:
            section_names.append(parent.name)
            parent = parent.parent
        return ' / '.join(reversed(section_names))

    def serializable_object(self):
        obj = {
            'pk': self.pk,
            'parent_id': self.parent_id,
            'name': self.name,
            'is_visible': __('yes') if self.is_visible else __('no'),
            'children': []
        }
        for child in self.get_children():
            obj['children'].append(child.serializable_object())
        return obj


class AbstractProductLabel(AbstractProductBase):
    class Meta:
        abstract = True
        app_label = 'shop'
        ordering = ['name']
        verbose_name_plural = _('product labels')
        verbose_name = _('product label')

    slug = models.SlugField(_('URL'), unique=True)
    image = VersatileImageField(_('Image'), upload_to='shop/label', null=True, blank=True)


class AbstractProduct(AbstractProductBase):
    class Meta:
        abstract = True
        app_label = 'shop'
        ordering = ['name']
        verbose_name_plural = _('products')
        verbose_name = _('product')

    slug = AutoSlugField(_('URL'), populate_from='name', unique=True)
    section = models.ForeignKey('shop.ProductSection', verbose_name=_('Section'), related_name='products',
                                on_delete=models.PROTECT)
    manufacturer = models.ForeignKey('shop.ProductManufacturer', verbose_name=_('Manufacturer'),
                                     related_name='manufacturer_products', null=True, blank=True,
                                     on_delete=models.PROTECT)
    labels = models.ManyToManyField('shop.ProductLabel', verbose_name=_('Labels'), related_name='label_products',
                                    blank=True)
    code = models.CharField(_('Code'), max_length=100, blank=True, default='')
    short_description = RteField(_('Short description'), blank=True, default='')
    residue = models.IntegerField(_('Residue'), default=1)
    price = models.DecimalField(verbose_name=_('Price'), decimal_places=2, max_digits=10, default=0)
    image = VersatileImageField(_('Image'), upload_to='shop/product/%Y/%m/%d', null=True, blank=True)


class AbstractProductImage(models.Model):
    class Meta:
        abstract = True
        app_label = 'shop'
        verbose_name = _('product picture')
        verbose_name_plural = _('product pictures')
        ordering = ['-date_created']

    product = models.ForeignKey('shop.Product', verbose_name=_('Product'), related_name='pictures',
                                on_delete=models.CASCADE)
    image = VersatileImageField(_('Image'), upload_to='shop/product_images/%Y/%m/%d')
    date_created = models.DateTimeField(auto_now_add=True)


class AbstractOrderShippingType(models.Model):
    class Meta:
        abstract = True
        app_label = 'shop'
        verbose_name = _('shipping type')
        verbose_name_plural = _('shipping types')
        ordering = ['sort']

    def __str__(self):
        return '%s (%s)' % (self.name, self.cost)

    slug = models.SlugField(_('URL'), unique=True)
    name = models.CharField(_('Name'), max_length=100)
    description = RteField(_('Description'), blank=True, default='')
    cost = models.DecimalField(_('Cost'), decimal_places=2, max_digits=10, default=0)
    is_enabled = models.BooleanField(_('Is enabled'), default=True)
    sort = models.IntegerField(default=1)
    date_created = models.DateTimeField(auto_now_add=True)


class AbstractOrderPaymentType(models.Model):
    class Meta:
        abstract = True
        app_label = 'shop'
        verbose_name = _('payment type')
        verbose_name_plural = _('payment types')
        ordering = ['sort']

    def __str__(self):
        return self.name

    slug = models.SlugField(_('URL'), unique=True)
    name = models.CharField(_('Name'), max_length=100)
    description = RteField(_('Description'), blank=True, default='')
    is_enabled = models.BooleanField(_('Is enabled'), default=True)
    sort = models.IntegerField(default=1)
    date_created = models.DateTimeField(auto_now_add=True)


class AbstractOrderStatus(models.Model):
    class Meta:
        abstract = True
        app_label = 'shop'
        verbose_name = _('order status')
        verbose_name_plural = _('order statuses')
        ordering = ['sort']

    def __str__(self):
        return self.name

    slug = models.SlugField(_('URL'), unique=True)
    name = models.CharField(_('Name'), max_length=100)
    description = RteField(_('Description'), blank=True, default='')
    is_enabled = models.BooleanField(_('Is enabled'), default=True)
    sort = models.IntegerField(default=1)
    date_created = models.DateTimeField(auto_now_add=True)


class AbstractOrder(models.Model):
    class Meta:
        abstract = True
        app_label = 'shop'
        verbose_name = _('order')
        verbose_name_plural = _('orders')
        ordering = ['-date_created']

    fullname = models.CharField(_('Fullname'), max_length=255)
    email = models.EmailField(_('E-mail'), null=True, blank=True)
    shipping_address = models.TextField(_('Shipping address'), null=True, blank=True)

    status = models.ForeignKey('shop.OrderStatus', verbose_name=_('Status'), null=True, blank=True,
                               on_delete=models.SET_NULL)
    shipping_type = models.ForeignKey('shop.OrderShippingType', verbose_name=_('Shipping type'), null=True, blank=True,
                                      on_delete=models.SET_NULL)
    shipping_cost = models.DecimalField(verbose_name=_('Cost'), decimal_places=2, max_digits=10, default=0)
    payment_type = models.ForeignKey('shop.OrderPaymentType', verbose_name=_('Payment type'), null=True, blank=True,
                                     on_delete=models.SET_NULL)

    is_paid = models.BooleanField(_('Is paid'), default=False)
    paid_sum = models.DecimalField(_('Paid sum'), decimal_places=2, max_digits=10, null=True, blank=True)
    discount_percent = models.DecimalField(_('Discount'), decimal_places=2, max_digits=5, default=0)

    date_created = models.DateTimeField(_('Date created'), auto_now_add=True)
    date_modified = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return '%s%s' % (_('Order #'), self.id)

    @property
    def order_number(self):
        return self.id

    @property
    def sum_value(self):
        model = get_model('shop', 'OrderDetail')
        sum_value = model.objects.filter(order=self).aggregate(
            sum_value=Sum(F('price') * F('quantity'), output_field=models.DecimalField()))['sum_value']
        if not sum_value:
            sum_value = 0
        return sum_value

    @property
    def discount_value(self):
        return self.sum_value * self.discount_percent / 100

    @property
    def quantity(self):
        model = get_model('shop', 'OrderDetail')
        quantity = model.objects.filter(order=self).aggregate(
            quantity=Sum('quantity'))['quantity']
        if not quantity:
            quantity = 0
        return quantity

    @property
    def total(self):
        return self.sum_value + self.shipping_cost - self.discount_value


class AbstractOrderDetail(models.Model):
    class Meta:
        abstract = True
        app_label = 'shop'
        verbose_name = _('order datail')
        verbose_name_plural = _('order datails')
        ordering = ['name']

    order = models.ForeignKey('shop.Order', verbose_name=_('Order'), related_name='details', on_delete=models.CASCADE)
    product = models.ForeignKey('shop.Product', null=True, on_delete=models.SET_NULL, verbose_name=_('Product'))
    code = models.CharField(_('Code'), max_length=255, blank=True)
    name = models.CharField(_('Name'), max_length=1024)
    price = models.DecimalField(_('Price'), decimal_places=2, max_digits=10, default=0)
    quantity = models.IntegerField(_('Quantity'), default=0)

    def __str__(self):
        return self.name

    @property
    def sum_value(self):
        return self.price * self.quantity



from django import forms
from django.conf import settings
from django.db.models import TextField
from versatileimagefield.datastructures import sizedimage


class RteField(TextField):
    def formfield(self, **kwargs):
        defaults = {'max_length': self.max_length, 'widget': forms.Textarea(attrs={'rows': 20, 'class': 'rte'})}
        defaults.update(kwargs)
        return super(RteField, self).formfield(**defaults)


def monkeypatch_versatile_image_field():
    """
    To bypass
    https://github.com/respondcreate/django-versatileimagefield/issues/59
    we monkey patch versatile image field to silent error when
    a specific setting is toggled
    """

    def dec(func):
        def wrapper(self, *args, **kwargs):
            try:
                return func(self, *args, **kwargs)
            except FileNotFoundError:
                if settings.ALLOW_MISSING_MEDIA:
                    return
                raise

        return wrapper

    kls = sizedimage.SizedImage
    kls.create_resized_image = dec(kls.create_resized_image)



from importlib import import_module

from django.apps import apps
from django.apps.config import MODELS_MODULE_NAME
from django.conf import settings
from django.core.exceptions import AppRegistryNotReady


def get_model(app_label, model_name):
    try:
        return apps.get_model(app_label, model_name)
    except AppRegistryNotReady:
        if apps.apps_ready and not apps.models_ready:
            app_config = apps.get_app_config(app_label)
            import_module('%s.%s' % (app_config.name, MODELS_MODULE_NAME))
            return apps.get_registered_model(app_label, model_name)
        else:
            raise


def get_form_class(app_label, form_class_name):
    module = import_module('%s.%s' % (app_label, getattr(settings, 'FORMS_MODULE_NAME', 'forms')))
    return getattr(module, form_class_name)


def is_model_registered(app_label, model_name):
    try:
        apps.get_registered_model(app_label, model_name)
    except LookupError:
        return False
    else:
        return True



from dartcms.apps.users.models import UserGroup
from dartcms.utils.loading import get_model
from django import forms
from django.utils.translation import ugettext_lazy as _
from django.forms import ModelForm

PageModule = get_model('pages', 'PageModule')
AdSection = get_model('ads', 'AdSection')


class PageForm(ModelForm):
    class Meta:
        model = get_model('pages', 'Page')
        exclude = ['url', 'date_created', 'date_changed', 'sort']
        widgets = {
            'module_params': forms.Select()
        }

        fieldsets = (
            ('main', {'fields': (
                'parent',
                'slug',
                'title',
                'header',
                'menu_name',
                'menu_url',
                'redirect_url',
                'module',
                'module_params',
            ), 'legend': _('Main')}),
            ('content', {'fields': (
                'before_content',
                'after_content',
            ), 'legend': _('Content')}),
            ('ads', {'fields': (
                'ad_section',
            ), 'legend': _('Ads')}),
            ('seo', {'fields': (
                'seo_keywords',
                'seo_description',
            ), 'legend': _('SEO')}),
            ('advanced', {'fields': (
                'security_type',
                'user_groups',
                'is_enabled',
                'is_in_menu',
            ), 'legend': _('Advanced')}))

    module = forms.ModelChoiceField(PageModule.objects.filter(is_enabled=True).exclude(slug='homepage'),
                                    empty_label=None, label=_('Module'))
    ad_section = forms.ModelChoiceField(AdSection.objects.filter(is_enabled=True), required=False, label=_('Ads'))
    user_groups = forms.ModelMultipleChoiceField(UserGroup.objects.all(), required=False, label=_('User groups'))

    def protect_homepage(self):
        del self.fields['is_enabled']
        del self.fields['redirect_url']
        del self.fields['module']
        del self.fields['parent']
        del self.fields['module_params']
        del self.fields['slug']

    def __init__(self, *args, **kwargs):
        request = kwargs.pop('request')
        super(PageForm, self).__init__(*args, **kwargs)

        if not request.user.is_superuser:
            del self.fields['security_type']
            del self.fields['user_groups']

        if self.instance.pk:
            if self.instance.module.slug == 'homepage':
                self.protect_homepage()
        else:
            self.fields['module'].initial = PageModule.objects.get(slug='page')



from dartcms.utils.fields import monkeypatch_versatile_image_field
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class FeedsConfig(AppConfig):
    name = 'dartcms.apps.feeds'
    verbose_name = _('Feeds')

    def ready(self):
        monkeypatch_versatile_image_field()



from dartcms import get_model
from dartcms.utils.config import DartCMSConfig
from dartcms.views import UpdateObjectView
from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_exempt

from .forms import ProductSectionForm
from .views import (AppendSectionView, GetTreeView, InsertSectionView,
                    MoveSectionView)

app_name = 'sections'

ProductSection = get_model('shop', 'ProductSection')

config = DartCMSConfig({
    'model': ProductSection,
    'parent_kwarg_name': 'catalog',
    'parent_model_fk': 'catalog_id',
    'grid': {
        'grid_columns': [
            {'field': 'name', 'width': '90%'},
            {'field': 'is_visible', 'width': '10%'},
        ],
        'template_name': 'dartcms/apps/shop/grid.html',
        'additional_grid_actions': [
            {'url': 'products', 'label': _('Products'),
             'kwarg_name': 'section', 'include_urls': 'dartcms.apps.shop.product.urls'}
        ]
    },
    'form': {
        'form_class': ProductSectionForm,
    },
})

insert_kwargs = config.form.copy()
insert_kwargs['template_name'] = 'dartcms/views/insert.html'

update_kwargs = config.form.copy()
update_kwargs['template_name'] = 'dartcms/views/update.html'

urlpatterns = config.get_urls(exclude=['insert', 'update']) + [
    url(r'^get-tree/$', csrf_exempt(GetTreeView.as_view(**config.grid)), name='get_tree'),
    url(r'^insert/$', InsertSectionView.as_view(**insert_kwargs), name='insert'),
    url(r'^append/$', csrf_exempt(AppendSectionView.as_view()), name='append'),
    url(r'^move/$', csrf_exempt(MoveSectionView.as_view()), name='move'),
    url(r'^update/(?P<pk>\d+)/$', UpdateObjectView.as_view(**update_kwargs), name='update'),
]



# Generated by Django 1.9.6 on 2016-07-05 05:21
from __future__ import unicode_literals

from dartcms import get_model
from django.db import migrations
from django.utils import translation
from django.utils.translation import gettext_lazy as _


def insert_homepage(apps, schema):
    from django.conf import settings
    translation.activate(settings.LANGUAGE_CODE)

    PageModule = get_model('pages', 'PageModule')
    Page = get_model('pages', 'Page')

    if PageModule.objects.filter(slug='homepage').exists():
        return

    homepage_module = PageModule.objects.create(
        slug='homepage',
        name=_('Homepage')
    )

    homepage = {
        'parent': None,
        'title': _('Homepage'),
        'slug': '',
        'url': '/',
        'module': homepage_module
    }
    Page.objects.create(**homepage)

    translation.deactivate()


def drop_homepage(apps, schema):
    PageModule = apps.get_model('pages', 'PageModule')
    Page = apps.get_model('pages', 'Page')

    Page.objects.filter(module__slug='homepage').delete()
    PageModule.objects.filter(slug='homepage').delete()


class Migration(migrations.Migration):
    dependencies = [
        ('pages', '0010_auto_20161214_0258'),
    ]

    operations = [
        migrations.RunPython(insert_homepage, drop_homepage)
    ]



# Generated by Django 1.9.6 on 2016-07-05 05:12
from __future__ import unicode_literals

import django.db.models.deletion
import django.db.models.manager
import mptt.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('ads', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title',
                 models.CharField(help_text='Shows inside the title tag', max_length=255, verbose_name='Title')),
                ('header', models.CharField(max_length=255, verbose_name='Page Header')),
                ('menu_name', models.CharField(default=b'', max_length=255, verbose_name='Menu name')),
                ('redirect_url',
                 models.CharField(blank=True, default=b'', max_length=255, verbose_name='URL for Redirect')),
                ('slug', models.SlugField(blank=True, default=b'', verbose_name='URL')),
                ('url', models.CharField(max_length=512)),
                ('sort', models.IntegerField(default=1)),
                ('module_params', models.CharField(blank=True, default=None, max_length=128, null=True,
                                                   verbose_name='Module parameters')),
                ('before_content', models.TextField(blank=True, default=b'', verbose_name='Before Content')),
                ('after_content', models.TextField(blank=True, default=b'', verbose_name='After Content')),
                ('seo_keywords', models.TextField(blank=True, default=b'', verbose_name='Keywords (meta keywords)')),
                ('seo_description',
                 models.TextField(blank=True, default=b'', verbose_name='Description (meta description)')),
                ('is_enabled', models.BooleanField(default=True, verbose_name='Is Enabled')),
                ('is_in_menu', models.BooleanField(default=True, verbose_name='Show in Menu')),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('lft', models.PositiveIntegerField(db_index=True, editable=False)),
                ('rght', models.PositiveIntegerField(db_index=True, editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(db_index=True, editable=False)),
                ('ad_section',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pages_page_related',
                                   to='ads.AdSection', verbose_name='Ads', null=True, blank=True, default=None)),
            ],
            options={
                'ordering': ['sort'],
                'abstract': False,
                'verbose_name': 'page',
                'verbose_name_plural': 'pages',
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name='PageModule',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(unique=True, verbose_name='Slug')),
                ('name', models.CharField(max_length=64, verbose_name='Name')),
                ('is_enabled', models.BooleanField(default=True, verbose_name='Is Enabled')),
            ],
            options={
                'ordering': ['id'],
                'verbose_name': 'module',
                'verbose_name_plural': 'modules',
            },
        ),
        migrations.AddField(
            model_name='page',
            name='module',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pages_page_related',
                                    to='pages.PageModule', verbose_name='Module'),
        ),
        migrations.AddField(
            model_name='page',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                                             related_name='children', to='pages.Page', verbose_name='Parent Page'),
        ),
        migrations.AlterUniqueTogether(
            name='page',
            unique_together=set([('module', 'slug')]),
        ),
    ]



# Generated by Django 1.9.7 on 2016-07-12 11:42
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('feeds', '0002_auto_20160706_0827'),
    ]

    operations = [
        migrations.AlterField(
            model_name='feeditem',
            name='feed',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='feeds.Feed', verbose_name='Feed'),
        ),
        migrations.AlterField(
            model_name='feeditem',
            name='picture',
            field=models.ImageField(blank=True, null=True, upload_to=b'feeds/%Y/%m/%d', verbose_name='Picture'),
        ),
    ]



from dartcms.utils.loading import get_model
from django.contrib import admin

AdPlace = get_model('ads', 'AdPlace')
AdSection = get_model('ads', 'AdSection')


class AdPlaceAdmin(admin.ModelAdmin):
    pass


admin.site.register(AdPlace, AdPlaceAdmin)


class AdSectionAdmin(admin.ModelAdmin):
    pass


admin.site.register(AdSection, AdSectionAdmin)



from django import forms

from .models import SiteUser


class UserForm(forms.ModelForm):
    class Meta:
        model = SiteUser
        exclude = ('password', 'last_login', 'date_joined', 'user_permissions', 'groups', 'is_staff', 'is_superuser')



from dartcms.utils.loading import get_model
from django.forms import ModelForm


class OrderDatailForm(ModelForm):
    class Meta:
        model = get_model('shop', 'OrderDetail')
        exclude = []



# Generated by Django 1.9.9 on 2016-11-30 12:07
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0008_auto_20161026_0711'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='pagemodule',
            options={'ordering': ['name'], 'verbose_name': 'module', 'verbose_name_plural': 'modules'},
        ),
    ]



from dartcms.views import (DeleteMultipleObjectView, DeleteObjectView,
                           GridView, InsertObjectView,
                           InsertObjectWithInlinesView, UpdateObjectView,
                           UpdateObjectWithInlinesView)
from django.conf.urls import include, url
from django.forms import modelform_factory
from django.utils.translation import ugettext as _

from .db import get_model_field_label, get_model_field_type


class DartCMSConfig(object):
    config = {}

    default_action_label = _('View Records')
    default_action_icon = 'next'

    def __init__(self, config):
        self.config = config

    @property
    def base(self):
        return {
            'parent_kwarg_name': self.config.get('parent_kwarg_name'),
            'parent_model_fk': self.config.get('parent_model_fk'),
            'parent_model': self.config.get('parent_model'),
            'model': self.config.get('model'),
        }

    def construct_grid_columns(self):
        model = self.config['model']
        for column in self.config['grid']['grid_columns']:
            if 'label' not in column:
                column['label'] = get_model_field_label(model, column['field'])
            if 'type' not in column:
                column['type'] = get_model_field_type(model, column['field'])

    def construct_additional_grid_actions(self):
        for action in self.config['grid']['additional_grid_actions']:
            if 'label' not in action:
                action['label'] = self.default_action_label
            if 'icon' not in action:
                action['icon'] = self.default_action_icon

    @property
    def grid(self):
        config = self.base

        if config['model']:
            if 'grid_columns' in self.config['grid']:
                self.construct_grid_columns()

        if 'additional_grid_actions' in self.config['grid']:
            self.construct_additional_grid_actions()

        config.update(self.config['grid'])
        return config

    @property
    def form(self):
        config = self.base
        if 'form' in self.config:
            config.update(self.config['form'])
        elif config.get('model'):
            config['form_class'] = modelform_factory(config['model'], exclude=[])
        return config

    def get_urls(self, exclude=None):
        if exclude is None:
            exclude = []
        urls = []

        if 'index' not in exclude:
            urls += [url(r'^$', GridView.as_view(**self.grid), name='index')]

        if 'insert' not in exclude:
            insert_view = InsertObjectWithInlinesView if 'inlines' in self.form else InsertObjectView
            urls += [url(r'^insert/$', insert_view.as_view(**self.form), name='insert')]

        if 'update' not in exclude:
            update_view = UpdateObjectWithInlinesView if 'inlines' in self.form else UpdateObjectView
            urls += [url(r'^update/(?P<pk>[0-9a-f-]+)/$', update_view.as_view(**self.form), name='update')]

        if 'delete' not in exclude:
            if not self.grid.get('single_select', True):
                urls += [
                    url(r'^delete/(?P<pks>[0-9a-f-,]+)/$', DeleteMultipleObjectView.as_view(**self.base), name='delete')]
            else:
                urls += [url(r'^delete/(?P<pk>[0-9a-f-]+)/$', DeleteObjectView.as_view(**self.base), name='delete')]

        if 'addition' not in exclude and 'additional_grid_actions' in self.grid:
            for action in self.grid.get('additional_grid_actions', []):
                if 'kwarg_name' in action:
                    pattern = r'^(?P<children_url>{url})/(?P<{kwarg_name}>[0-9a-f-]+)/'
                else:
                    pattern = r'^(?P<children_url>{url})/'

                if 'include_urls' in action:
                    urls += [url(pattern.format(**action),
                                 include(action['include_urls'], namespace=action['url']))]
                else:
                    urls += [url(pattern.format(**action), action['view'], name=action['url'])]

        return urls



from dartcms import get_model
from dartcms.utils.config import DartCMSConfig

from .forms import OrderDatailForm

app_name = 'order_details'

Order = get_model('shop', 'OrderDetail')

config = DartCMSConfig({
    'model': Order,
    'parent_kwarg_name': 'order',
    'parent_model_fk': 'order_id',
    'grid': {
        'grid_columns': [
            {'field': 'name', 'width': '60%'},
            {'field': 'price', 'width': '20%'},
            {'field': 'quantity', 'width': '20%'},
        ],
        'base_grid_actions': ['update', 'delete'],
    },
    'form': {
        'form_class': OrderDatailForm
    }
})

urlpatterns = config.get_urls(exclude=['insert'])



from django.conf import settings
from django.conf.urls import include, url
from django.views.i18n import JavaScriptCatalog

js_info_dict = {
    'packages': ('dartcms',),
}

app_name = 'dartcms'

urlpatterns = [
    url(r'^i18n/', include('django.conf.urls.i18n')),
    url(r'^jsi18n/$', JavaScriptCatalog.as_view(), name='javascript-catalog'),

    url(r'^', include('dartcms.apps.dashboard.urls', namespace='dashboard')),
    url(r'^auth/', include('dartcms.apps.auth.urls', namespace='auth')),

    url(r'^filemanager/', include('dartcms.apps.filemanager.urls', namespace='filemanager')),
    url(r'^dict-(?P<module_slug>[a-z_-]{3,50})/',
        include('dartcms.apps.dicts.urls', namespace='dicts')),
]

if 'dartcms.apps.users.groups' in settings.INSTALLED_APPS:
    urlpatterns += [url(r'^user-groups/', include('dartcms.apps.users.groups.urls', namespace='user_groups')), ]

if 'dartcms.apps.siteusers' in settings.INSTALLED_APPS:
    urlpatterns += [url(r'^site-users/', include('dartcms.apps.siteusers.urls', namespace='siteusers')), ]

if 'dartcms.apps.users' in settings.INSTALLED_APPS:
    urlpatterns += [url(r'^cms-users/', include('dartcms.apps.users.urls', namespace='users')), ]

if 'dartcms.apps.pages' in settings.INSTALLED_APPS:
    urlpatterns += [url(r'^sitemap/', include('dartcms.apps.pages.urls', namespace='pages')), ]

if 'dartcms.apps.feeds' in settings.INSTALLED_APPS:
    urlpatterns += [url(r'^feeds/', include('dartcms.apps.feeds.urls', namespace='feeds')), ]

if 'dartcms.apps.sitesettings' in settings.INSTALLED_APPS:
    urlpatterns += [url(r'^sitesettings/', include('dartcms.apps.sitesettings.urls', namespace='sitesettings')), ]

if 'dartcms.apps.ads' in settings.INSTALLED_APPS:
    urlpatterns += [
        url(r'^ad/', include('dartcms.apps.ads.ad.urls', namespace='ad')),
        url(r'^adplace/', include('dartcms.apps.ads.adplace.urls', namespace='adplace')),
        url(r'^adsection/', include('dartcms.apps.ads.adsection.urls', namespace='adsection')), ]

if 'dartcms.apps.feedback' in settings.INSTALLED_APPS:
    urlpatterns += [url(r'^feedback/', include('dartcms.apps.feedback.urls', namespace='feedback')), ]

if 'dartcms.apps.shop' in settings.INSTALLED_APPS:
    urlpatterns += [
        url(r'^shop-catalog/', include('dartcms.apps.shop.catalog.urls', namespace='shop-catalog')),
        url(r'^shop-orders/', include('dartcms.apps.shop.order.urls', namespace='shop-orders')), ]

additional_apps = getattr(settings, 'DARTCMS_ADDITIONAL_APPS_URLPATTERNS', [])

if additional_apps:
    additional_patterns = []
    for app in additional_apps:
        additional_patterns.append(url(r'^%s/' % app[0], include(app[1], namespace=app[2])))

    urlpatterns += additional_patterns



# Generated by Django 1.9.7 on 2016-07-14 12:49
from __future__ import unicode_literals

import autoslug.fields
import dartcms.utils.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shop', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProductManufacturer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('description', dartcms.utils.fields.RteField(blank=True, default=b'', verbose_name='Description')),
                ('seo_keywords', models.TextField(blank=True, default=b'', verbose_name='Keywords (meta keywords)')),
                ('seo_description', models.TextField(blank=True, default=b'', verbose_name='Keywords (meta keywords)')),
                ('is_visible', models.BooleanField(default=True, verbose_name='Is Visible')),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('slug', autoslug.fields.AutoSlugField(editable=False, populate_from=b'name', unique=True, verbose_name='URL')),
                ('image', models.ImageField(blank=True, null=True, upload_to=b'shop/producer', verbose_name='Image')),
            ],
            options={
                'ordering': ['name'],
                'abstract': False,
                'verbose_name': 'product manufacturer',
                'verbose_name_plural': 'product manufacturers',
            },
        ),
        migrations.AddField(
            model_name='product',
            name='manufacturer',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='manufacturer_products', to='shop.ProductManufacturer', verbose_name='Section'),
        ),
    ]



# Generated by Django 2.0.1 on 2018-01-23 11:13

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('feeds', '0006_auto_20170829_0515'),
    ]

    operations = [
        migrations.AlterField(
            model_name='feed',
            name='type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='feeds', to='feeds.FeedType', verbose_name='Type'),
        ),
    ]



from dartcms.views import (DeleteObjectView, GridView, InsertObjectView,
                           UpdateObjectView)
from django.conf.urls import url

app_name = 'dicts'

urlpatterns = [
    url(r'^$', GridView.as_view(search=['name']), name='index'),
    url(r'^insert/$', InsertObjectView.as_view(), name='insert'),
    url(r'^update/(?P<pk>\d+)/$', UpdateObjectView.as_view(), name='update'),
    url(r'^delete/(?P<pk>\d+)/$', DeleteObjectView.as_view(), name='delete'),
]



from django.contrib.sites.shortcuts import get_current_site
from django.http import Http404
from django.shortcuts import get_object_or_404
from django.template.defaultfilters import urlencode
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from django.views.generic import ListView, RedirectView

from mptt.utils import get_cached_trees

from dartcms.utils.loading import get_model
from dartcms.views import (DeleteObjectView, GridView, InsertObjectView,
                           JSONView, UpdateObjectView)
from dartcms.views.mixins import JSONResponseMixin, ModulePermissionsMixin

Page = get_model('pages', 'Page')
PageModule = get_model('pages', 'PageModule')


class PageTreeView(GridView):
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if self.request.GET.get('search'):
            context['search_params_str'] = '?%s' % "&".join('{!s}={!s}'.format(
                k, urlencode(v)) for k, v in self.request.GET.items() if k in ['title', 'url', 'module', 'search'])
            context['parent_str'] = _('Search Results')
        return context


class GetTreeView(GridView, JSONResponseMixin):
    model = Page

    def post(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def render_to_response(self, context, **response_kwargs):
        return self.render_to_json_response(context, safe=False, **response_kwargs)

    def get_search_res(self):
        qs = self.model.objects.all()
        if self.request.GET.get('title'):
            qs = qs.filter(title__icontains=self.request.GET.get('title'))
        if self.request.GET.get('module'):
            qs = qs.filter(module_id=self.request.GET.get('module'))
        if self.request.GET.get('url'):
            qs = qs.filter(url__icontains=self.request.GET.get('url'))

        if qs.exists():
            qs = qs.get_ancestors(include_self=True)
            home = get_cached_trees(qs)[0]
            return [home.serializable_object_with_children]
        else:
            return []

    def get_data(self, context):
        if self.request.GET.get('search'):
            return self.get_search_res()

        parent_id = self.request.POST.get('id')
        qs = self.object_list.filter(parent_id=parent_id)
        return [obj.serializable_object for obj in qs]


class PageFormKwargsMixin:
    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['request'] = self.request
        return kwargs


class SecurityMixin:
    def has_perm(self):
        if self.request.user.is_superuser:
            return True

        security_type = self.check_object.get_security_type()
        if security_type == 'GROUPS_ONLY':
            return len(
                set(self.request.user.user_groups.all()).intersection(set(self.check_object.user_groups.all()))) > 0
        return True

    def check_object(self):
        raise NotImplemented

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(**kwargs)
        context['has_perm'] = self.has_perm()
        return context


class InsertPageView(SecurityMixin, PageFormKwargsMixin, InsertObjectView):
    @cached_property
    def check_object(self):
        return self.model.objects.get(pk=self.request.GET['parent'])

    def get_initial(self):
        return {'parent': self.check_object}

    def form_valid(self, form):
        obj = self.save_object(form)
        data = {
            'pk': obj.pk,
            'title': obj.title,
            'module': obj.module.name,
            'url': obj.url
        }
        return self.render_to_json_response({'result': True, 'action': 'INSERT', 'data': data})


class UpdatePageView(SecurityMixin, PageFormKwargsMixin, UpdateObjectView):
    @cached_property
    def check_object(self):
        return self.object

    def form_valid(self, form):
        obj = form.save()
        data = {
            'title': obj.title,
            'module': obj.module.name,
            'url': obj.url
        }
        return self.render_to_json_response({'result': True, 'action': 'UPDATE', 'data': data})


class TreeActionView(SecurityMixin, ModulePermissionsMixin, JSONView):
    @cached_property
    def check_object(self):
        return Page.objects.get(pk=self.request.GET.get('source'))

    def get_pages(self):
        target_id = self.request.GET.get('target')
        source_id = self.request.GET.get('source')

        if target_id and source_id:
            target = Page.objects.get(pk=target_id)
            source = Page.objects.get(pk=source_id)

            return target, source

        raise Http404


class AppendPageView(TreeActionView):
    def get_data(self, context):
        if not self.has_perm():
            return {'result': False}

        target, source = self.get_pages()
        source.move_to(target, position='last-child')
        return {'result': True}


class MovePageView(TreeActionView):
    def get_data(self, context):
        if not self.has_perm():
            return {'result': False}

        position = self.request.GET.get('position')
        target, source = self.get_pages()
        if target.parent:
            source.move_to(target, position=position)
            return {'result': True}
        return {'result': False}


class LoadModuleParamsView(ModulePermissionsMixin, JSONView):
    def get_data(self, context):
        try:
            module = PageModule.objects.get(pk=self.request.GET.get('selected_module'))
        except PageModule.DoesNotExist:
            module = None

        if module and module.related_model:
            model_path = module.related_model.split('.')
            app = model_path[0]
            model = model_path[1]
            try:
                RelatedModel = get_model(app, model)
                data = RelatedModel.objects.all()
            except (LookupError, ImportError):
                data = []
        else:
            data = []

        page_pk = self.request.GET.get('pk', 0)

        if page_pk != 0:
            page = Page.objects.get(pk=page_pk)
        else:
            page = None

        params = []

        for item in data:
            if page is not None and page.module == module and str(page.module_params) == str(item.pk):
                params.append({'label': str(item), 'value': item.pk, 'selected': True})
            else:
                params.append({'label': str(item), 'value': item.pk})

        params = sorted(params, key=lambda k: k['label'])

        return {
            'result': True,
            'data': params
        }


class DeletePageView(SecurityMixin, DeleteObjectView):
    @cached_property
    def check_object(self):
        return self.object


class OpenUrlRedirectView(RedirectView):
    permanent = False

    def get_redirect_url(self, *args, **kwargs):
        domain = get_current_site(self.request)
        url = get_object_or_404(Page, pk=self.kwargs.get('pk')).url
        return '//%s%s' % (domain, url)



from django import forms
from django.contrib.auth import authenticate, get_user_model
from django.utils.translation import ugettext_lazy as _

User = get_user_model()


class LoginForm(forms.Form):
    username = forms.CharField(label=User._meta.get_field(User.USERNAME_FIELD).verbose_name.title())
    password = forms.CharField(widget=forms.PasswordInput, label=_('Password'))
    next = forms.CharField(widget=forms.HiddenInput, required=False)

    def __init__(self, *args, **kwargs):
        self.user_cache = None
        super(LoginForm, self).__init__(*args, **kwargs)

    def clean(self):
        data = self.cleaned_data
        if 'username' not in data or 'password' not in data:
            raise forms.ValidationError(_('All fields are required'))

        username = self.cleaned_data.get('username')
        password = self.cleaned_data.get('password')

        if username and password:
            user = authenticate(username=username,
                                password=password)
            if user is None:
                raise forms.ValidationError(_('Incorrect username of password'))
            if not user.is_staff:
                raise forms.ValidationError(_('Access denied'))

            self.user_cache = user
        return self.cleaned_data

    def get_user(self):
        return self.user_cache
